from PyQt6.QtWidgets import QHBoxLayout, QFileDialog, QApplication, QMessageBox, QWidget, QComboBox, QVBoxLayout, QPushButton, QLabel, QLineEdit, QProgressBar
from PyQt6.QtCore import QThread, pyqtSignal, QTimer
from uglypty.Library.collector_lib import *
from uglypty.Library.util import cryptonomicon
import sqlite3
import datetime
import traceback

class Worker(QThread):
    progress = pyqtSignal(int)
    success = pyqtSignal(str)
    failure = pyqtSignal(str)
    skipped = pyqtSignal(str)

    def __init__(self, max_job_size, query, output_dir, command, devices, device_type, username, password):
        super().__init__()
        self.run_complete = False
        self.max_job_size = max_job_size
        self.query = query
        self.output_dir = output_dir
        self.command = command
        self.devices = devices
        self.device_type = device_type
        self.username = username
        self.password = password
        self.timestamp = datetime.datetime.now().strftime('%Y-%m-%d %H:%M')

    def run(self):

        try:
            with ProcessPoolExecutor() as process_executor:
                for i in range(0, len(self.devices), self.max_job_size):
                    job_devices = self.devices[i:i + self.max_job_size]

                    with ThreadPoolExecutor() as thread_executor:
                        for device in job_devices:
                            device['device_type'] = self.device_type
                            device['username'] = self.username
                            device['password'] = self.password
                            device['timestamp'] = self.timestamp
                            device['command'] = self.command

                        job_results = thread_executor.map(lambda device: ssh_to_device(device), job_devices)

                        for idx, result in enumerate(job_results):
                            filename = job_devices[idx]['display_name']
                            # Update progress regardless of job result
                            if not self.run_complete:
                                self.progress.emit(i + self.max_job_size)
                            else:
                                self.progress.emit(100)
                            if result is not None:
                                save_result_to_file(self.output_dir, filename, result)
                                # Emit success or failure signal based on the result
                                print(f"Result: {result.get('status')}")

                                if result['status'] == 'success':
                                    self.success.emit(filename)
                                elif result['status'] == 'skipped':
                                    self.skipped.emit(filename)
                                else:
                                    self.failure.emit(filename)
                            else:
                                self.failure.emit(filename)
            print(f"Run Complete")
        except Exception as e:
            print(f"Error occurred: {e}")
            traceback.print_exc()
        finally:
            QTimer.singleShot(1000, lambda: self.progress.emit(100))

class CollectorForm(QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("Netmiko Collector")
        self.success_count = {}
        self.failure_count = {}
        self.skipped_count = {}
        self.setFixedWidth(400)
        # Create widgets
        self.layout = QVBoxLayout()

        self.label_max_jobs = QLabel("Max Job Size:")
        self.edit_max_jobs = QLineEdit()
        self.edit_max_jobs.setText(max_job_size)

        # Create new horizontal layout
        self.yaml_layout = QHBoxLayout()

        # Create and add label
        self.yaml_label = QLabel("YAML File:")
        self.yaml_layout.addWidget(self.yaml_label)

        # Create and add line edit
        self.yaml_lineedit = QLineEdit("./sessions/sessions.yaml")
        self.yaml_layout.addWidget(self.yaml_lineedit)

        # Create and add push button
        def open_file_dialog():
            filename, _ = QFileDialog.getOpenFileName(None, "Open YAML File", "", "YAML Files (*.yaml)")
            if filename:
                self.yaml_lineedit.setText(filename)

        self.yaml_button = QPushButton("Browse...")
        self.yaml_button.clicked.connect(open_file_dialog)
        self.yaml_button.setStyleSheet("background-color: #8B6508; color: white;")
        self.yaml_layout.addWidget(self.yaml_button)


        # Add the new hbox layout to your existing layout
        self.layout.addLayout(self.yaml_layout)

        self.label_query = QLabel("SQL Query:")
        self.edit_query = QLineEdit()
        self.edit_query.setText(query_default)
        self.label_device_type = QLabel("Netmiko Type:")
        self.device_type_combo = QComboBox()
        self.device_type_combo.addItems(["cisco_xe", "cisco_nxos", "cisco_ios","cisco_xr","cloudgenix_ion","fortinet","hp_procurve","hp_comware", "juniper_junos", "junos", "arista_eos", "linux"])

        self.label_output_dir = QLabel("Output Directory:")
        self.edit_output_dir = QLineEdit()
        self.edit_output_dir.setText(str(output_dir))

        self.select_folder_button = QPushButton("Select Folder")
        self.select_folder_button.clicked.connect(self.select_folder)
        self.select_folder_button.setStyleSheet("background-color: #8B6508; color: white;")
        self.directory_layout = QHBoxLayout()
        self.directory_layout.addWidget(self.edit_output_dir)
        self.directory_layout.addWidget(self.select_folder_button)

        self.label_command = QLabel("Command to Run:")
        self.edit_command = QLineEdit()
        self.edit_command.setText("show version")

        self.label_user = QLabel("User:")
        self.combo_user = QComboBox()

        self.progress_bar = QProgressBar()
        self.progress_bar.setVisible(False)
        self.button_start = QPushButton("Start")
        self.button_start.setStyleSheet("background-color: #006400; color: white;")

        self.label_success = QLabel("Success: 0")
        self.label_failure = QLabel("Failures: 0")
        self.label_skipped = QLabel("Skipped: 0")

        # Add widgets to layout
        self.layout.addWidget(self.label_max_jobs)
        self.layout.addWidget(self.edit_max_jobs)
        self.layout.addWidget(self.label_device_type)
        self.layout.addWidget(self.device_type_combo)
        self.layout.addWidget(self.label_query)
        self.layout.addWidget(self.edit_query)
        self.layout.addWidget(self.label_user)
        self.layout.addWidget(self.combo_user)
        self.layout.addLayout(self.directory_layout)
        self.layout.addWidget(self.label_command)
        self.layout.addWidget(self.edit_command)
        self.layout.addWidget(self.progress_bar)
        self.layout.addWidget(self.label_success)
        self.layout.addWidget(self.label_failure)
        self.layout.addWidget(self.label_skipped)
        self.layout.addWidget(self.button_start)

        self.setLayout(self.layout)

        self.button_start.clicked.connect(self.start)
        # self.worker.skipped.connect(self.increment_skipped)
        self.load_users()

    def select_folder(self):
        directory = QFileDialog.getExistingDirectory(self, "Select Output Directory")

        if directory:  # if user didn't pick a directory don't change anything
            self.edit_output_dir.setText(directory)  # sets the text to the selected directory

    def get_creds(self, user_id):
        conn = sqlite3.connect("settings.sqlite")
        cursor = conn.cursor()
        cursor.execute("SELECT username, password FROM creds WHERE id = ?", (user_id,))
        record = cursor.fetchone()
        encrypted_password = record[1]
        username = record[0]
        conn.close()

        # Decrypt the password
        decrypted_password = cryptonomicon(encrypted_password)

        return username, decrypted_password

    def load_users(self):
        conn = sqlite3.connect("settings.sqlite")
        cursor = conn.cursor()
        cursor.execute("SELECT id, username FROM creds")
        rows = cursor.fetchall()
        conn.close()

        for row in rows:
            id, username = row
            self.combo_user.addItem(username, id)

    def start(self):
        self.progress_bar.setValue(0)
        self.progress_bar.setVisible(True)
        # command = self.edit_command.text()
        max_job_size = int(self.edit_max_jobs.text())
        query = self.edit_query.text()
        output_dir = self.edit_output_dir.text()
        command = str(self.edit_command.text()).strip()
        device_type = self.device_type_combo.currentText()
        yaml_file = self.yaml_lineedit.text()  # Get yaml file from lineedit

        devices = fetch_devices(yaml_file, query)
        self.progress_bar.setMaximum(len(devices))
        username, password = self.get_creds(self.combo_user.currentData())
        self.worker = Worker(max_job_size, query, output_dir, command, devices, device_type, username, password)
        self.worker.progress.connect(self.progress_bar.setValue)
        self.worker.success.connect(self.increment_success)
        self.worker.failure.connect(self.increment_failure)
        self.worker.skipped.connect(self.increment_skipped)
        self.worker.finished.connect(self.complete)  # Connect to 'finished' signal
        self.worker.start()

    def complete(self):
        self.progress_bar.setValue(100)

        self.progress_bar.setVisible(False)

        self.notify("Done", "Worker has finished execution.")
        print("Worker has finished execution.")

    def increment_success(self, hostname):
        self.success_count[hostname] = self.success_count.get(hostname, 0) + 1
        total_successes = sum(self.success_count.values())
        self.label_success.setText(f"Successes: {total_successes}")

    def increment_failure(self, hostname):
        self.failure_count[hostname] = self.failure_count.get(hostname, 0) + 1
        total_failures = sum(self.failure_count.values())
        self.label_failure.setText(f"Failures: {total_failures}")

    def increment_skipped(self, hostname):
        self.skipped_count[hostname] = self.skipped_count.get(hostname, 0) + 1
        total_skipped = sum(self.skipped_count.values())
        self.label_skipped.setText(f"Skipped: {total_skipped}")

    def notify(self, message, info):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Icon.Information)
        msg.setText(info)
        msg.setWindowTitle(message)
        msg.setStandardButtons(QMessageBox.StandardButton.Ok)
        retval = msg.exec()

if __name__ == "__main__":
    app = QApplication([])
    import qdarktheme


    qdarktheme.setup_theme("dark")
    window = CollectorForm()
    window.show()
    app.exec()

    '''netmiko device types by value:
    Supported SSH device_type values
    a10
    accedian
    adtran_os
    adva_fsp150f2
    adva_fsp150f3
    alcatel_aos
    alcatel_sros
    allied_telesis_awplus
    apresia_aeos
    arista_eos
    arris_cer
    aruba_os
    aruba_osswitch
    aruba_procurve
    audiocode_66
    audiocode_72
    audiocode_shell
    avaya_ers
    avaya_vsp
    broadcom_icos
    brocade_fastiron
    brocade_fos
    brocade_netiron
    brocade_nos
    brocade_vdx
    brocade_vyos
    calix_b6
    casa_cmts
    cdot_cros
    centec_os
    checkpoint_gaia
    ciena_saos
    cisco_asa
    cisco_ftd
    cisco_ios
    cisco_nxos
    cisco_s200
    cisco_s300
    cisco_tp
    cisco_viptela
    cisco_wlc
    cisco_xe
    cisco_xr
    cloudgenix_ion
    coriant
    dell_dnos9
    dell_force10
    dell_isilon
    dell_os10
    dell_os6
    dell_os9
    dell_powerconnect
    dell_sonic
    dlink_ds
    eltex
    eltex_esr
    endace
    enterasys
    ericsson_ipos
    ericsson_mltn63
    ericsson_mltn66
    extreme
    extreme_ers
    extreme_exos
    extreme_netiron
    extreme_nos
    extreme_slx
    extreme_tierra
    extreme_vdx
    extreme_vsp
    extreme_wing
    f5_linux
    f5_ltm
    f5_tmsh
    flexvnf
    fortinet
    generic
    generic_termserver
    hillstone_stoneos
    hp_comware
    hp_procurve
    huawei
    huawei_olt
    huawei_smartax
    huawei_vrp
    huawei_vrpv8
    ipinfusion_ocnos
    juniper
    juniper_junos
    juniper_screenos
    keymile
    keymile_nos
    linux
    mellanox
    mellanox_mlnxos
    mikrotik_routeros
    mikrotik_switchos
    mrv_lx
    mrv_optiswitch
    netapp_cdot
    netgear_prosafe
    netscaler
    nokia_srl
    nokia_sros
    oneaccess_oneos
    ovs_linux
    paloalto_panos
    pluribus
    quanta_mesh
    rad_etx
    raisecom_roap
    ruckus_fastiron
    ruijie_os
    sixwind_os
    sophos_sfos
    supermicro_smis
    teldat_cit
    tplink_jetstream
    ubiquiti_edge
    ubiquiti_edgerouter
    ubiquiti_edgeswitch
    ubiquiti_unifiswitch
    vyatta_vyos
    vyos
    watchguard_fireware
    yamaha
    zte_zxros
    zyxel_os
    '''


# from Library.settings_json import usettings
from PyQt6 import QtCore, QtGui, QtWidgets
from PyQt6.QtWidgets import QAbstractItemView
from uglypty.Library.model2 import Cred
from uglypty.Library.db import get_by_dbquery
from sqlalchemy import create_engine
from sqlalchemy.orm import Session
from creds_dlg import Ui_dlgCreds
import json
from uglypty.Library.util import cryptonomicon, encrypt


# used to make table selection by row only
class ReadOnlyDelegate(QtWidgets.QStyledItemDelegate):
    def createEditor(self, parent, option, index):
        print('createEditor event fired')
        return

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        self.formIsDirty = False
        MainWindow.setObjectName("TerminalsUI")
        MainWindow.resize(883, 299)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.twCreds = QtWidgets.QTableWidget(self.centralwidget)
        self.twCreds.setObjectName("twCreds")
        self.twCreds.setColumnCount(3)
        self.twCreds.setRowCount(0)
        # enable context menu's
        self.twCreds.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.CustomContextMenu)
        self.twCreds.customContextMenuRequested['QPoint'].connect(self.tw_right_click)
        self.twCreds.horizontalHeader().setDefaultSectionSize(180)
        item = QtWidgets.QTableWidgetItem()
        self.twCreds.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.twCreds.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.twCreds.setHorizontalHeaderItem(2, item)
        self.verticalLayout.addWidget(self.twCreds)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 883, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuCreds")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionExit = QtGui.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionExit.triggered.connect(lambda: sys.exit())
        self.actionAdd = QtGui.QAction(MainWindow)
        self.actionAdd.setObjectName("actionAdd")
        self.actionAdd.triggered.connect(lambda: self.creds_add())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionAdd)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("TerminalsUI", "Device Credentials"))
        item = self.twCreds.horizontalHeaderItem(0)
        item.setText(_translate("TerminalsUI", "id"))
        item = self.twCreds.horizontalHeaderItem(1)
        item.setText(_translate("TerminalsUI", "username"))
        item = self.twCreds.horizontalHeaderItem(2)
        item.setText(_translate("TerminalsUI", "password"))
        self.menuFile.setTitle(_translate("TerminalsUI", "File"))
        self.actionExit.setText(_translate("TerminalsUI", "Exit"))
        self.actionAdd.setText(_translate("TerminalsUI", "Add Record"))

        # make whole table read only
        delegateTw = ReadOnlyDelegate(self.twCreds)
        self.twCreds.setItemDelegate(delegateTw)

        # select by row only
        self.twCreds.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.twCreds.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
        self.load_db_creds()


    def tw_right_click(self):
        selected = self.twCreds.currentIndex()
        engine = create_engine(f'sqlite:///settings.sqlite', echo=False)
        engine.connect()
        sqlite_session = Session(bind=engine)
        # print(f"Selected Row: {selected.row()}!")
        if selected.row() != -1:
            # refBinding created when item created. col 0 is container
            try:
                print(f"selected: {self.twCreds.item(selected.row(),0).refBinding['Username']}")
            except:
                self.notify("data error", f"error reading refbinding")
                return
            top_menu = QtWidgets.QMenu()
            menu = top_menu.addMenu("Menu")
            _edit = menu.addAction("&Edit ...")
            _delete = menu.addAction("&Delete ...")
            action = menu.exec(QtGui.QCursor.pos())
            if action == _edit:
                self.creds_edit()

            if action == _delete:
                self.creds_delete()

    def confirm(self, message, info):
        msgBox = QtWidgets.QMessageBox()
        msgBox.setIcon(QtWidgets.QMessageBox.Icon.Information)
        msgBox.setText(info)
        msgBox.setWindowTitle(message)
        msgBox.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok | QtWidgets.QMessageBox.StandardButton.Cancel)
        returnValue = msgBox.exec()
        return returnValue

    def creds_delete(self):
        selected = self.twCreds.currentIndex()
        if selected.row() != -1:
            delete_item = self.twCreds.item(selected.row(), 0)
            # print(f"deleting selected user: {delete_item.refBinding['Username']} ID: {delete_item.refBinding['id']}")
            try:
                # now save to DB
                engine = create_engine(f'sqlite:///settings.sqlite', echo=False)
                engine.connect()
                sqlite_session = Session(bind=engine)
                sql = f"select * from creds where id = {delete_item.refBinding['id']}"
                sql_result = get_by_dbquery(Cred, sqlite_session, sql)

                if len(sql_result) == 0:
                    self.notify("DB Error", f"Error processing database: No Device Records")
                else:
                    response = self.confirm("Delete?", f"Delete creds: {self.twCreds.item(selected.row(), 1).text()}")
                    if response == QtWidgets.QMessageBox.StandardButton.Ok:
                        print(
                            f"deleting selected user: {delete_item.refBinding['Username']} ID: {delete_item.refBinding['id']}")
                        sqlite_session.delete(sql_result[0])
                        sqlite_session.commit()
                        self.twCreds.setRowCount(0)
                        self.load_db_creds()


            except Exception as e:
                print(e)
                self.notify("Database Error", f"Error deleting creds: {e}")

    def creds_add(self):
        stub_item = {'id': 'New', 'Username': '', 'Password': ''}
        dlg = QtWidgets.QDialog()
        dlg.setModal(True)
        ui = Ui_dlgCreds()
        ui.setupUi(dlg, stub_item, "ADD")
        # ui.mode = "ADD"
        dlg.show()
        result = dlg.exec()
        if result == 1:
            encrypted_password = encrypt(stub_item)
            stub_item['Password'] = encrypted_password.decode('utf-8')
            print(f"Item to add: {json.dumps(stub_item)}")
            try:
                # now save to DB
                engine = create_engine(f'sqlite:///settings.sqlite', echo=False)
                engine.connect()
                sqlite_session = Session(bind=engine)
                cur = engine.execute("select id from creds order by id desc")
                first_record = cur.first()
                if first_record is None:
                    # empty table, start with 1
                    last_id = 0
                else:
                    last_id = first_record[0]
                # last_id = cur.first()[0]
                new_record = Cred(
                    id=last_id + 1,
                    Username=stub_item['Username'],
                    Password=stub_item['Password']
                )
                sqlite_session.add(new_record)
                sqlite_session.commit()
                sqlite_session.close()
                self.twCreds.setRowCount(0)
                self.load_db_creds()
            except Exception as e:
                print(e)
                self.notify("Database Error", f"Error processing creds: {e}")


    def creds_edit(self):
        selected = self.twCreds.currentIndex()
        if selected.row() != -1:
            print(f"editing selected row {selected.row()}")
            edit_item = self.twCreds.item(selected.row(), 0)
            try:
                print(edit_item.refBinding)
                dlg = QtWidgets.QDialog()
                dlg.setModal(True)
                ui = Ui_dlgCreds()
                ui.setupUi(dlg, edit_item.refBinding,"EDIT")
                dlg.show()
                result = dlg.exec()
                if result == 1:
                    self.formIsDirty = True
                    encrypted_password = encrypt(edit_item.refBinding['Password'])
                    edit_item.refBinding['Password'] = encrypted_password.decode('utf-8')
                    # verify item refBinding in col 0 of selected row is updated
                    print(f"Data to save: {self.twCreds.item(selected.row(), 0).refBinding}")

                    try:
                        # now save to DB
                        engine = create_engine(f'sqlite:///settings.sqlite', echo=False)
                        engine.connect()
                        sqlite_session = Session(bind=engine)
                        sql = f"select * from creds where id = {edit_item.refBinding['id']}"
                        sql_result = get_by_dbquery(Cred, sqlite_session, sql)
                        current_row = 0
                        if len(sql_result) == 0:
                            self.notify("DB Error", f"Error processing database: No Device Records")
                        else:
                            print(sql_result[0])
                            sql_result[0].Username = edit_item.refBinding['Username']
                            sql_result[0].Password = edit_item.refBinding['Password']
                            sqlite_session.commit()
                        sqlite_session.close()
                        # reload table for data refresh
                        self.twCreds.setRowCount(0)
                        self.load_db_creds()
                    except Exception as e:
                        print(e)
                        self.notify("Database Error", f"Error processing creds: {e}")


            except Exception as e:
                self.notify("edit error", "Error opening row, no refBinding data")


    def notify(self, message, info):
        msg = QtWidgets.QMessageBox()
        msg.setIcon(QtWidgets.QMessageBox.Icon.Information)
        msg.setText(info)
        msg.setWindowTitle(message)
        msg.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok)
        retval = msg.exec()

    def load_db_creds(self):
        engine = create_engine(f'sqlite:///settings.sqlite', echo=False)
        engine.connect()
        sqlite_session = Session(bind=engine)
        try:
            sql = "select * from creds"
            sql_result = get_by_dbquery(Cred, sqlite_session, sql)
            current_row = 0
            if len(sql_result) == 0:
                self.notify("New DB", f"No Credentials Records, start by adding new creds")
            else:
                self.twCreds.setRowCount(len(sql_result))
                cred_list = []
                map_cmdb = {
                    0: "id",
                    1: "Username",
                    2: "Password"
                }
                for cred in sql_result:
                    # print(cred)
                    tcred = cred.toDict()
                    print(tcred)
                    for i in range(3):
                        newitem = QtWidgets.QTableWidgetItem(str(tcred[map_cmdb[i]]))
                        newitem.refBinding = tcred
                        newitem.refKey = tcred['id']
                        newitem.refRow = current_row
                        self.twCreds.setItem(current_row, i, newitem)

                    current_row += 1

            sqlite_session.close()

        except Exception as e:
            self.notify("Database Error", f"Error loading Creds table: {e}")

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MainWindow = QtWidgets.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    MainWindow.show()
    sys.exit(app.exec())



import sys
import time
import os
import json

from PyQt6.QtCore import QSize, QCoreApplication, QUrl, QMetaObject, QTimer
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QMainWindow, QMessageBox
from PyQt6.QtWebEngineWidgets import QWebEngineView
from PyQt6.QtWebEngineCore import QWebEngineProfile
from PyQt6.QtWebChannel import QWebChannel
from uglypty.Library.sshschemahandler import WebEngineUrlSchemeHandler
from uglypty.Library.sshshell import Backend


class Ui_Terminal(QWidget):
    """
    Terminal class extending QWidget to enable SSH connections in a Qt widget.
    """

    def __init__(self, connect_info, parent=None):
        """
        Initialization function for the Terminal class.

        :param connect_info: a dictionary that includes SSH credentials.
        :param parent: parent widget if any.
        """
        super().__init__(parent)
        if isinstance(connect_info, tuple):
            print("This is a tuple")
            connect_info = connect_info[0]
            print(connect_info)

            self.mode = "application"
            # Do something with tuple
        elif isinstance(connect_info, dict):
            print("This is a dict, should be good")
            self.mode = "standalone"
            # Do something with dict
        else:
            print("Unknown data type")
        self.host = connect_info.get('host')
        self.port = connect_info.get('port', '22')
        self.username = connect_info.get('username')
        self.password = connect_info.get('password')
        self.pkey_path = connect_info.get('pkey_path')
        self.log_filename = connect_info.get('log_filename')
        self.theme = connect_info.get('theme')
        if self.theme == "light_dark":
            xterm_theme = "light"
        elif self.theme == "dark_light":
            xterm_theme = "dark"
        else:
            xterm_theme = self.theme

        print(f"Xterm.js theme: {xterm_theme}")
        self.div_height = 0

        self.setupUi(self)

    def setupUi(self, term):
        """
        Setups UI for the terminal widget.

        :param term: terminal widget instance.
        """
        term.setObjectName("term")
        QMetaObject.connectSlotsByName(term)
        layout = QVBoxLayout()
        self.handler = WebEngineUrlSchemeHandler()
        QWebEngineProfile.defaultProfile().installUrlSchemeHandler(b"file", self.handler)
        self.channel = QWebChannel()
        try:
            # For key-based auth
            if self.pkey_path:
                self.backend = Backend(host=self.host, port=self.port, username=self.username, key_path=self.pkey_path, parent_widget=self)
            else:
                self.backend = Backend(host=self.host, port=self.port, username=self.username, password=self.password, parent_widget=self)

            self.channel.registerObject("backend", self.backend)
        except:
            return
        self.view = QWebEngineView()
        self.view.page().setWebChannel(self.channel)
        self.div_height = 0
        self.webview_size = self.view.size()

        self.view.resizeEvent = self.handle_resize_event
        self.view.loadFinished.connect(self.handle_load_finished)
        self.backend.send_output.connect(
            lambda data: self.view.page().runJavaScript(f"window.handle_output({json.dumps(data)})"))

        base_dir = os.path.dirname(os.path.abspath(__file__))
        if self.mode == "standalone":
            # filename = "../static/qtsshcon.html"
            filename = os.path.join(base_dir, '../static/qtsshcon.html')
            print(f"Loading html: {filename}")
        else:
            if self.theme == "light" or self.theme == "light_dark":
                # filename = "./static/qtsshcon_light.html"
                filename = os.path.join(base_dir, '../static/qtsshcon_light.html')
            else:
                # filename = "./static/qtsshcon.html"
                filename = os.path.join(base_dir, '../static/qtsshcon.html')
        html_to_load = os.path.abspath(filename)
        print(f"Trying to load... {html_to_load}")
        self.view.load(QUrl.fromLocalFile(html_to_load))
        layout.addWidget(self.view)
        term.setLayout(layout)
        self.retranslateUi(term)

    def update_div_height(self):
        """
        Updates the div height of the terminal.
        """
        script = f"document.getElementById('terminal').style.height = '{self.div_height}px';"
        self.view.page().runJavaScript(script)

    def handle_load_finished(self):
        """
        Handles actions after the web page load has finished.
        """
        self.div_height = self.view.size().height() - 30
        self.update_div_height()
        current_size = self.view.size()
        new_size = QSize(current_size.width(), current_size.height() + 1)
        self.view.resize(new_size)
        print("loaded..")
        QTimer.singleShot(0, self.delayed_method)

    def handle_resize_event(self, event):
        """
        Handles resize events of the terminal.

        :param event: event object containing event details.
        """
        self.div_height = self.view.size().height() - 30
        if self.view.size() != self.webview_size:
            self.webview_size = self.view.size()
            self.update_div_height()

    def retranslateUi(self, term):
        """
        Retranslates the UI based on the current locale.

        :param term: terminal widget instance.
        """
        _translate = QCoreApplication.translate
        term.setWindowTitle(_translate("term", "term"))

    def delayed_method(self):
        """
        This method will be called after the web page load has finished.
        """
        try:
            print(f"Buffer: {json.dumps(self.initial_buffer)}")
            banner = json.dumps(self.initial_buffer).replace('"', '')
            self.view.page().runJavaScript(f"term.write('{banner}');")
        except:
            pass



    def notify(self, message, info):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Icon.Information)
        msg.setText(info)
        msg.setWindowTitle(message)
        msg.setStandardButtons(QMessageBox.StandardButton.Ok)
        retval = msg.exec()

if __name__ == "__main__":
    """
    Main function. Initializes and runs the application.
    """
    try:
        app = QApplication(sys.argv)
        mainWin = QMainWindow()
        mainWin.resize(800, 400)

        terminal = Ui_Terminal({"host": "10.0.0.12",
                                "username": "admin",
                                "password": "cisco",
                                }, mainWin)
        mainWin.setCentralWidget(terminal)
        mainWin.setWindowTitle("PyQt6 - SSH Widget")
        mainWin.show()
        sys.exit(app.exec())

    except Exception as e:
        print(f"Exception in main: {e}")



from PyQt6.QtCore import pyqtSignal, QThread
import serial

class SerialReaderThread(QThread):
    data_ready = pyqtSignal(str)

    def __init__(self, serial):
        super().__init__()
        self.serial = serial

    def run(self):
        while True:
            if self.serial.is_open:
                try:
                    data = self.serial.readline().decode('utf-8')
                    if data:  # only emit data if something is read
                        self.data_ready.emit(data)
                except Exception as e:
                    print(f"Error while reading from serial port: {e}")
            else:
                print("Serial port is not open...")
                break



from PyQt6.QtCore import pyqtSignal, QThread
import os

class ShellReaderThread(QThread):
    data_ready = pyqtSignal(str)

    def __init__(self, channel, buffer, parent_widget):
        super().__init__()
        self.channel = channel
        self.intial_buffer = buffer
        self.parent_widget = parent_widget
        if parent_widget.log_filename is not None:
            self.log_filename = parent_widget.log_filename
        else:
            self.log_filename = "../logs/session.log"

        # Ensure log directory exists
        os.makedirs(os.path.dirname(self.log_filename), exist_ok=True)

    def log_data(self, data):
        # Make sure that a log filename was provided
        if self.log_filename is not None:
            try:
                # Attempt to encode and decode the data as UTF-8 to ensure it's valid
                data = data.encode('utf-8', 'ignore').decode('utf-8', 'ignore')
                data = data.replace('\r\n', '\n').replace('\r', '\n')
                with open(self.log_filename, 'a', encoding='utf-8') as log_file: # 'a' option will append data to the end of the file
                    # log_file.write(data + os.linesep)
                    log_file.write(data)
            except UnicodeError:
                print("Failed to encode data as UTF-8.")


    def run(self):
        while True:
            if not self.channel.closed:
                try:
                    data = self.channel.recv(1024)

                    if data:
                        data_decoded = data.decode()
                        # Log data that is being received
                        self.log_data(data_decoded)

                        # for debugging
                        if self.intial_buffer == "":
                            self.intial_buffer = bytes(data).decode('utf-8')
                            self.parent_widget.initial_buffer = bytes(data).decode('utf-8')

                        self.data_ready.emit(data_decoded)
                except Exception as e:
                    print(f"Error while reading from channel: {e}")
                    self.log_data(f"Error while reading from channel: {e}")
            else:
                print("Channel closed...")
                self.log_data("Channel closed...")
                break



from concurrent.futures import ThreadPoolExecutor
import socket
import os
from PyQt6.QtCore import QThread, pyqtSignal


class PortScannerThread(QThread):
    textReady = pyqtSignal(str)  # Signal emitted when there is new text to show on the UI
    scanComplete = pyqtSignal(list)  # Signal emitted when scanning is complete

    def __init__(self, ip_list, port_list, thread_num, show_refused, wait_time, stop_after_count):
        super(PortScannerThread, self).__init__()
        self.ip_list = ip_list
        self.port_list = port_list
        self.thread_num = thread_num
        self.show_refused = show_refused
        self.wait_time = wait_time
        self.stop_after_count = stop_after_count
        self.open_results = []

    def run(self):
        found_count = 0  # Keep track of how many open ports are found

        with ThreadPoolExecutor(max_workers=self.thread_num) as executor:
            # Create a flat list of (ip, port) combinations
            tasks = [(ip, port) for ip in self.ip_list for port in self.port_list]

            # Use executor.map() to map ping_port function across all (ip, port) combinations
            results = list(executor.map(lambda p: self.ping_port(*p), tasks))

            # Filter out None results (you can adapt ping_port to return some status)
            self.open_results = [r for r in results if r is not None]

        self.scanComplete.emit(self.open_results)  # Emit signal to indicate that scanning is complete

    def ping_port(self, ip, port):
        self.textReady.emit(f"scanning... {ip}:{port}")
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(self.wait_time)
        status = sock.connect_ex((ip, port))
        if status == 0:
            self.textReady.emit(f'{ip}:{port} OPEN')
            return (ip, port)  # Return the open port information
        elif status not in [35, 64, 65] and self.show_refused:
            self.textReady.emit(f'{ip}:{port} ERRNO {status}, {os.strerror(status)}')
            return None



import os
import yaml
from ttp import ttp

class IPtoName():
    def __init__(self):
        print("IPtoName")

    def run(self):
        # List of TTP templates
        ttp_templates = [
            '''Device ID: {{ device_id }}
          IP address: {{ ip }}
        Platform: {{ platform | ORPHRASE }},  Capabilities: {{ capabilities | ORPHRASE}}
        Interface: {{ local_port | ORPHRASE }},  Port ID (outgoing port): {{ remote_port | ORPHRASE}}''',
            '''System Name: {{ device_id }}
            IPv4 Address: {{ ip }}
        Platform: {{ platform | ORPHRASE }},  Capabilities: {{ capabilities | ORPHRASE}}
        Interface: {{ local_port | ORPHRASE }},  Port ID (outgoing port): {{ remote_port | ORPHRASE}}'''
        ]

        # Initialize lookup dictionary and failed list
        ip_device_lookup = {}
        failed_files = []

        def parse_file(filename, templates):
            with open(f"{folder_to_scan}/{filename}", 'r') as f:
                data = f.read()

            for template in templates:
                try:
                    parser = ttp(data=data, template=template)
                    parser.parse()
                    result = parser.result()
                    print(f"Parser result for file {filename} with template: {result}")  # Debugging line

                    if result and isinstance(result[0][0], list):
                        return result[0][0]
                    elif result and isinstance(result[0][0], dict):
                        return [result[0][0]]

                except Exception as e:
                    print(f"Parser exception for file {filename} with template: {e}")

            return None  # Return None if all templates fail

        # Traverse the directory for .txt files
        folder_to_scan = './cli_cdp_output'
        for filename in os.listdir(folder_to_scan):
            if filename.endswith('.txt'):
                parsed_data = parse_file(filename, ttp_templates)
                print(f"Parsed data for file {filename}: {parsed_data}")  # Debugging line

                if parsed_data:
                    for entry in parsed_data:
                        if 'ip' in entry and 'device_id' in entry:
                            ip_device_lookup[entry['ip']] = entry['device_id']
                else:
                    failed_files.append(filename)

        def deduplicate_dict(self, input_dict):
            deduped_dict = {}
            for key, value in input_dict.items():
                if "SEP" not in value:
                    if key not in deduped_dict:
                        deduped_dict[key] = value
                    else:
                        # Handle duplicates, for now, let's keep the last value
                        deduped_dict[key] = value
            return deduped_dict

        ip_device_lookup = deduplicate_dict(self, ip_device_lookup)

        # Write the lookup to a YAML file
        with open('./Output/ip_device_lookup.yaml', 'w') as f:
            yaml.dump(ip_device_lookup, f)

        # Write the failed files to failed.res
        with open('failed.res', 'w') as f:
            for file in failed_files:
                f.write(f"{file}\n")

if __name__ == "__main__":
    iptoname = IPtoName()
    iptoname.run()


import sqlite3

conn = sqlite3.connect('settings.sqlite')
c = conn.cursor()

c.execute('''
          CREATE TABLE "creds" (
	"id" INTEGER NOT NULL  ,
	"username" TEXT NULL  ,
	"password" TEXT NULL
, "displayname"	TEXT)
          ''')

conn.commit()
print("settings.sqlite created")



import os
from PyQt6.QtWidgets import QVBoxLayout, QPushButton, QWidget, QFileDialog, QTreeWidget, QTreeWidgetItem
from PyQt6.QtGui import QIcon, QColor, QBrush
from PyQt6.QtCore import Qt, QTimer, pyqtSignal
import yaml

from PyQt6.QtWidgets import QHBoxLayout


class FileTree(QWidget):
    file_double_clicked = pyqtSignal(tuple)  # Emit a tuple containing the file path and line number

    def __init__(self, parent=None):
        super(FileTree, self).__init__(parent)
        self.current_folder = './Capture'  # or another default directory
        current_dir = os.path.dirname(os.path.abspath(__file__))
        self.layout = QVBoxLayout(self)

        # Create a horizontal layout for the buttons
        self.button_layout = QHBoxLayout()

        self.button = QPushButton("Select Folder", self)
        self.button.clicked.connect(self.select_folder)
        self.button_layout.addWidget(self.button)

        # Add a Refresh button
        self.refresh_button = QPushButton(self)
        self.refresh_button.setMaximumWidth(30)
        self.refresh_button.setIcon(QIcon(f"{current_dir}/../icons/refresh.png"))  # Set the icon
        self.refresh_button.clicked.connect(self.refresh_tree)
        self.button_layout.addWidget(self.refresh_button)

        # Add the horizontal layout to the main layout
        self.layout.addLayout(self.button_layout)

        self.tree = QTreeWidget(self)
        self.tree.setHeaderHidden(True)
        self.tree.itemDoubleClicked.connect(self.on_item_double_clicked)
        self.layout.addWidget(self.tree)

        print(f"current folder: {current_dir}")
        self.folder_icon = QIcon(f"{current_dir}/../icons/folder.png")
        self.folder_open_icon = QIcon(f"{current_dir}/../icons/folder_open.png")
        self.file_icon = QIcon(f"{current_dir}/../icons/file.png")
        try:
            self.load_directory_structure("./Capture")  # Changed this line to use the new method
        except:
            self.load_directory_structure("./")
    # New method to refresh the tree
    def refresh_tree(self):
        self.load_folder(self.current_folder)
    def select_folder(self):
        folder = QFileDialog.getExistingDirectory(self, "Select Folder")
        if folder:
            self.current_folder = folder
            self.load_directory_structure(folder)  # Changed this line to use the new method

    def load_directory_structure(self, folder, parent_item=None):
        # Create separate lists for directories and files
        dirs = []
        files = []

        for entry in os.scandir(folder):
            if entry.name.startswith('.'):
                continue
            if entry.is_dir():
                dirs.append(entry)
            else:
                files.append(entry)

        # Sort the directories and files
        dirs.sort(key=lambda e: e.name.lower())
        files.sort(key=lambda e: e.name.lower())

        # If parent_item is None, add items to the tree, otherwise add them as children of the parent_item
        if parent_item is None:
            self.tree.clear()
            add_item = self.tree
        else:
            parent_item.takeChildren()  # Remove existing children
            parent_item.setIcon(0, self.folder_open_icon)  # Set the icon for the opened folder
            add_item = parent_item

        # Add directories and files to the tree widget
        for entries in [dirs, files]:
            for entry in entries:
                icon = self.folder_icon if entry.is_dir() else self.file_icon

                # If the entry is a file and the extension is .yaml, load the status from the file
                if entry.is_file() and entry.name.endswith('.yaml'):
                    try:
                        with open(entry.path, 'r') as file:
                            content = yaml.safe_load(file)
                            if 'status' in content:
                                item = QTreeWidgetItem(add_item)
                                item.setIcon(0, icon)
                                item.setText(0, entry.name)
                                item.setData(0, Qt.ItemDataRole.UserRole, entry.path)  # Store the full path
                                if content['status'] == 'failure':
                                    item.setForeground(0, QBrush(QColor('red')))
                                elif content['status'] == 'skipped':
                                    item.setForeground(0, QBrush(QColor('yellow')))
                    except yaml.YAMLError as error:
                        print(f"Error parsing YAML file {entry.path}: {error}")
                else:  # If the entry is not a .yaml file or a directory, add it normally
                    item = QTreeWidgetItem(add_item)
                    item.setIcon(0, icon)
                    item.setText(0, entry.name)
                    item.setData(0, Qt.ItemDataRole.UserRole, entry.path)  # Store the full path

        if parent_item:
            QTimer.singleShot(100, lambda: parent_item.setExpanded(True))

        if parent_item is None:
            self.tree.expandAll()

    def load_folder(self, folder, search_string=None):
        self.tree.clear()
        if search_string is not None and search_string != "":
            matching_entries = self.search_folder(folder, search_string)

            for entry in matching_entries:
                self.add_entry_to_tree(entry)
        else:
            self.load_directory_structure(folder)  # Changed this line to use the new method

    # ... Rest of the code


    def search_folder(self, folder, search_string=None):
        matching_entries = []

        for root, dirs, files in os.walk(folder):
            for name in dirs + files:
                entry_path = os.path.join(root, name)
                if os.path.isfile(entry_path):
                    if "__" not in entry_path:
                        with open(entry_path, 'r', encoding='utf-8',  errors='ignore') as file:
                            lines = file.readlines()
                            i = 0
                            if search_string is None:
                                search_string = ""
                            for line in lines:
                                if search_string not in line:
                                    i += 1
                                    continue


                                matching_entries.append((entry_path, i))
                                print(f"Found match: {entry_path}")
                                i += 1
                else:
                    matching_entries.append((entry_path, None))

        return matching_entries

    def add_entry_to_tree(self, entry):
        path, line_number = entry
        dirs = path.split(os.sep)

        parent_item = None
        current_path = ""

        for dir in dirs:
            current_path = os.path.join(current_path, dir)

            found = False
            for i in range(self.tree.topLevelItemCount()):
                if self.tree.topLevelItem(i).text(0) == dir:
                    parent_item = self.tree.topLevelItem(i)
                    found = True
                    break
            if not found and parent_item is not None:
                for i in range(parent_item.childCount()):
                    if parent_item.child(i).text(0) == dir:
                        parent_item = parent_item.child(i)
                        found = True
                        break

            if not found:
                if parent_item is None:
                    parent_item = QTreeWidgetItem(self.tree)
                else:
                    parent_item = QTreeWidgetItem(parent_item)
                parent_item.setText(0, dir)
                parent_item.setData(0, Qt.ItemDataRole.UserRole, (current_path, line_number))

    def on_item_double_clicked(self, item, column):
        data = item.data(column, Qt.ItemDataRole.UserRole)
        if isinstance(data, tuple) and len(data) == 2:
            path, line_number = data
            if os.path.isfile(path):
                self.file_double_clicked.emit((path, line_number))
        elif isinstance(data, str):
            path = data
            if os.path.isfile(path):
                # Emit a signal with the path of the double-clicked file
                self.file_double_clicked.emit((path, None))
            elif os.path.isdir(path):
                # Load the directory
                self.load_directory_structure(path, item)

        else:
            print("don't know what to do")




from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QTextCursor, QTextFormat, QColor, QKeySequence, QTextCharFormat, QTextOption
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QHBoxLayout, QPlainTextEdit, QPushButton, QFileDialog, \
    QListWidget, QLabel
from diff_match_patch import diff_match_patch


class ReadOnlyEditor(QPlainTextEdit):
    def __init__(self):
        super().__init__()

    def keyPressEvent(self, event):
        if event.matches(QKeySequence.StandardKey.Paste):
            return
        super().keyPressEvent(event)

class App(QWidget):
    def __init__(self, parent=None):
        super().__init__()
        self.initUI()
        self.parent = parent

    def initUI(self):
        self.resize(800,400)
        layout = QVBoxLayout()
        hlayout = QHBoxLayout()

        self.leftEditor = ReadOnlyEditor()
        self.rightEditor = ReadOnlyEditor()
        self.leftEditor.setWordWrapMode(QTextOption.WrapMode.NoWrap)
        self.rightEditor.setWordWrapMode(QTextOption.WrapMode.NoWrap)
        self.leftLineNumbers = QListWidget()
        self.leftLineNumbers.setFixedWidth(30)
        self.rightLineNumbers = QListWidget()
        self.rightLineNumbers.setFixedWidth(30)

        hlayout.addWidget(self.leftLineNumbers)
        hlayout.addWidget(self.leftEditor)
        hlayout.addWidget(self.rightLineNumbers)
        hlayout.addWidget(self.rightEditor)

        legendLayout = QHBoxLayout()
        self.addLegendItem(legendLayout, "#8B6508", "Left Only")
        self.addLegendItem(legendLayout, "#8B0000", "Right Only")
        self.addLegendItem(legendLayout, "#006400", "Changed")

        layout.addLayout(hlayout)
        layout.addLayout(legendLayout)

        layout.addLayout(hlayout)

        # Hide the scrollbars in the QListWidgets
        self.leftLineNumbers.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.leftLineNumbers.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.rightLineNumbers.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.rightLineNumbers.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)

        # Connect the scroll bars for synchronized scrolling
        self.leftEditor.verticalScrollBar().valueChanged.connect(self.syncScroll)
        self.rightEditor.verticalScrollBar().valueChanged.connect(self.syncScroll)

        # Connect the scroll bars for line number synchronization
        self.leftEditor.verticalScrollBar().valueChanged.connect(self.syncLineNumberScroll)
        self.rightEditor.verticalScrollBar().valueChanged.connect(self.syncLineNumberScroll)

        btnOpenLeft = QPushButton('Open Left')
        btnOpenLeft.clicked.connect(self.openLeftFile)
        btnOpenRight = QPushButton('Open Right')
        btnOpenRight.clicked.connect(self.openRightFile)
        btnPasteLeft = QPushButton('Paste to Left')
        btnPasteLeft.clicked.connect(self.pasteToLeft)
        btnPasteRight = QPushButton('Paste to Right')
        btnPasteRight.clicked.connect(self.pasteToRight)
        btnCompare = QPushButton('Compare')
        btnCompare.clicked.connect(self.compareTexts)
        btnCompare.setStyleSheet("background-color: #234f96; color: white;")  # Dark forest green background and white text

        btnLayout = QHBoxLayout()
        btnLayout.addWidget(btnOpenLeft)
        btnLayout.addWidget(btnOpenRight)
        btnLayout.addWidget(btnPasteLeft)
        btnLayout.addWidget(btnPasteRight)
        btnLayout.addWidget(btnCompare)

        layout.addLayout(btnLayout)

        self.setLayout(layout)
        self.setWindowTitle('UglyDiff2')
        self.show()

    def syncScroll(self, value):
        sender = self.sender()
        if sender == self.leftEditor.verticalScrollBar():
            self.rightEditor.verticalScrollBar().setValue(value)
        elif sender == self.rightEditor.verticalScrollBar():
            self.leftEditor.verticalScrollBar().setValue(value)

    def syncLineNumberScroll(self, value):
        sender = self.sender()
        if sender == self.leftEditor.verticalScrollBar():
            self.leftLineNumbers.verticalScrollBar().setValue(value)
        elif sender == self.rightEditor.verticalScrollBar():
            self.rightLineNumbers.verticalScrollBar().setValue(value)

    def addLegendItem(self, layout, color, text):
        label = QLabel()
        label.setText(text)

        label.setStyleSheet(f"background-color: {color}; color: white; padding: 5px;")
        layout.addWidget(label)

    def updateLineNumbers(self, editor, listWidget):
        listWidget.clear()
        blockCount = editor.blockCount()
        for i in range(1, blockCount + 1):
            listWidget.addItem(str(i))

    def openLeftFile(self):
        filePath, _ = QFileDialog.getOpenFileName(self, "Open Left File", "", "All Files (*);;Text Files (*.txt)")
        if filePath:
            with open(filePath, 'r') as f:
                self.leftEditor.setPlainText(f.read())
            self.updateLineNumbers(self.leftEditor, self.leftLineNumbers)

    def openRightFile(self):
        filePath, _ = QFileDialog.getOpenFileName(self, "Open Right File", "", "All Files (*);;Text Files (*.txt)")
        if filePath:
            with open(filePath, 'r') as f:
                self.rightEditor.setPlainText(f.read())
            self.updateLineNumbers(self.rightEditor, self.rightLineNumbers)

    def pasteToLeft(self):
        clipboard = QApplication.clipboard()
        self.leftEditor.setPlainText(clipboard.text())
        self.updateLineNumbers(self.leftEditor, self.leftLineNumbers)

    def pasteToRight(self):
        clipboard = QApplication.clipboard()
        self.rightEditor.setPlainText(clipboard.text())
        self.updateLineNumbers(self.rightEditor, self.rightLineNumbers)



    # ... (your existing imports and code)

    def compareTexts(self):
        text1 = self.leftEditor.toPlainText().strip()
        text2 = self.rightEditor.toPlainText().strip()

        dmp = diff_match_patch()
        diffs = dmp.diff_main(text1, text2)
        dmp.diff_cleanupSemantic(diffs)  # Optional: clean up the diffs for better display

        left_cursor = self.leftEditor.textCursor()
        right_cursor = self.rightEditor.textCursor()

        # Define different formats for different types of changes
        left_only_format = QTextCharFormat()
        left_only_format.setBackground(QColor("#8B6508"))
        left_only_format.setForeground(QColor("white"))

        right_only_format = QTextCharFormat()
        right_only_format.setBackground(QColor("#8B0000"))
        right_only_format.setForeground(QColor("white"))

        changed_format = QTextCharFormat()
        changed_format.setBackground(QColor("#006400"))
        changed_format.setForeground(QColor("white"))

        left_pos = 0
        right_pos = 0

        for op, data in diffs:
            if op == 0:  # No change
                left_pos += len(data)
                right_pos += len(data)
            elif op == -1:  # Deletion (present in text1 but not text2)
                left_cursor.setPosition(left_pos)
                left_cursor.setPosition(left_pos + len(data), QTextCursor.MoveMode.KeepAnchor)
                left_cursor.mergeCharFormat(left_only_format)
                left_pos += len(data)
            elif op == 1:  # Insertion (present in text2 but not text1)
                right_cursor.setPosition(right_pos)
                right_cursor.setPosition(right_pos + len(data), QTextCursor.MoveMode.KeepAnchor)
                right_cursor.mergeCharFormat(right_only_format)
                right_pos += len(data)

        self.leftEditor.setTextCursor(left_cursor)
        self.rightEditor.setTextCursor(right_cursor)

    # ... (your other code)


if __name__ == '__main__':
    print(f"Debug webengine here: http://127.0.0.1:9222/")
    print("cli python uglyeditor.py --webEngineArgs --remote-debugging-port=9222")
    import sys

    app = QApplication(sys.argv)
    app.setStyle("fusion")
    ex = App()
    sys.exit(app.exec()) # comment



from PyQt6 import QtCore, QtGui, QtWidgets
from uglypty.Library.util import cryptonomicon, encrypt
import json
class Ui_dlgCreds(QtWidgets.QDialog):
    def setupUi(self, dlgCreds, data, mode):
        self.me = dlgCreds
        self.mode = mode
        self.data = data
        dlgCreds.setObjectName("dlgCreds")
        dlgCreds.resize(417, 155)
        self.verticalLayout = QtWidgets.QVBoxLayout(dlgCreds)
        self.verticalLayout.setObjectName("verticalLayout")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label_0 = QtWidgets.QLabel(dlgCreds)
        self.label_0.setObjectName("label_0")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_0)
        self.leDisplayName = QtWidgets.QLineEdit(dlgCreds)
        self.leDisplayName.setObjectName("leDisplayName")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.leDisplayName)

        self.label = QtWidgets.QLabel(dlgCreds)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.leID = QtWidgets.QLineEdit(dlgCreds)
        self.leID.setObjectName("leID")
        self.leID.setReadOnly(True)
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.leID)
        self.label_2 = QtWidgets.QLabel(dlgCreds)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.leUsername = QtWidgets.QLineEdit(dlgCreds)
        self.leUsername.setObjectName("leUsername")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.leUsername)
        self.label_3 = QtWidgets.QLabel(dlgCreds)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        self.lePassword = QtWidgets.QLineEdit(dlgCreds)
        self.lePassword.setObjectName("lePassword")
        self.lePassword.setEchoMode(QtWidgets.QLineEdit.EchoMode.Password)
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lePassword)
        self.label_4 = QtWidgets.QLabel(dlgCreds)
        self.label_4.setText("")
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_4)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.lblMode = QtWidgets.QLabel(dlgCreds)
        self.lblMode.setObjectName("lblMode")
        self.horizontalLayout.addWidget(self.lblMode)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.formLayout.setLayout(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout)
        self.verticalLayout.addLayout(self.formLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(dlgCreds)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(dlgCreds)
        self.buttonBox.accepted.connect(dlgCreds.accept) # type: ignore
        self.buttonBox.rejected.connect(dlgCreds.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(dlgCreds)

    def retranslateUi(self, dlgCreds):
        _translate = QtCore.QCoreApplication.translate
        dlgCreds.setWindowTitle(_translate("dlgCreds", "Automation Credentials"))
        self.label_0.setText(_translate("dlgCreds", "Display Name: "))
        self.label.setText(_translate("dlgCreds", "id"))
        self.label_2.setText(_translate("dlgCreds", "Username: "))
        self.label_3.setText(_translate("dlgCreds", "Password:"))
        self.lblMode.setText(_translate("dlgCreds", "Edit"))

        self.buttonBox.accepted.connect(self.okSaveChanges) # type: ignore
        self.buttonBox.rejected.connect(dlgCreds.reject) # type: ignore

        if self.mode == "EDIT":
            self.leID.setText(str(self.data['id']))
            self.leUsername.setText(str(self.data['Username']))
            self.leDisplayName.setText(str(self.data['DisplayName']))
            try:
                unencrypted_payload = cryptonomicon(self.data['Password'])

                self.lePassword.setText(unencrypted_payload)
            except Exception as e:
                self.notify("Decryption Error", "Unable to decrypt pw, possibly due to crypto.key change\nenter a new password")


            self.lblMode.setText(_translate("dlgCreds", "               Save?"))

        if self.mode == "ADD":
            self.lblMode.setText(_translate("dlgCreds", "               Save New Creds?"))

    def notify(self, message, info):
        msg = QtWidgets.QMessageBox()
        msg.setIcon(QtWidgets.QMessageBox.Icon.Information)
        msg.setText(info)
        msg.setWindowTitle(message)
        msg.setStandardButtons(QtWidgets.QMessageBox.StandardButton.Ok)
        retval = msg.exec()

    def okSaveChanges(self):
        print("Saving form to data passed in")
        self.data['Username'] = str(self.leUsername.text())
        self.data['Password'] = str(self.lePassword.text())
        # add displayname
        self.data['DisplayName'] = str(self.leDisplayName.text())
        self.me.accept()

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    sample_data = {'id': 4, 'Username': 'cisco', 'Password': 'gAAAAABkuEzZI7vL5itNNojW_6ifhdWx-RjRRsWGInj_7gqYuwWw5R1GjDJ7KuMuEI2ZUNlP30Vp4lRcP1VMAVT5Dab-e7FDAg==', 'DisplayName': "testing dlg"}
    dlgCreds = QtWidgets.QDialog()
    ui = Ui_dlgCreds()
    ui.setupUi(dlgCreds, sample_data, "EDIT")
    dlgCreds.show()
    sys.exit(app.exec())



import re
import sys
from ipaddress import ip_address, ip_network

from uglypty.uglyplugin_portscanlight.PortScannerThread import PortScannerThread
from uglypty.uglyplugin_portscanlight.portscan import PortScan
from uglypty.uglyplugin_portscanlight.pscan_help import help_content
from PyQt6.QtWidgets import (
    QApplication, QWidget, QVBoxLayout, QLineEdit,
    QPushButton, QPlainTextEdit, QComboBox, QFormLayout,
    QCheckBox, QMenuBar, QMenu, QTextBrowser, QLabel
)
from PyQt6.QtCore import QProcess, Qt, pyqtSignal, pyqtSlot
from PyQt6.QtGui import QAction
import os

class PortScannerGUI(QWidget):
    output_received = pyqtSignal(str)
    on_scan_complete = pyqtSignal(list)

    def __init__(self):
        super().__init__()
        self.portScannerThread = None

        self.init_ui()

    def init_ui(self):
        self.setMinimumWidth(500)
        layout = QVBoxLayout()
        self.setWindowTitle("TCP Port Scanner Lite")
        menu_bar = QMenuBar()
        help_menu = QMenu('Help', self)
        portscanner_action = QAction('PortScanner', self)
        about_action = QAction('About', self)

        portscanner_action.triggered.connect(self.show_portscanner_info)
        help_menu.addAction(portscanner_action)
        help_menu.addAction(about_action)
        menu_bar.addMenu(help_menu)

        layout.addWidget(menu_bar)  # Add the menu bar to the layout

        form_layout = QFormLayout()

        self.ip_input = QLineEdit()
        self.port_input = QLineEdit()
        self.thread_num_input = QComboBox()
        self.show_refused_input = QCheckBox()
        self.wait_time_input = QLineEdit()
        self.stop_after_count_input = QLineEdit()

        self.thread_num_input.addItems([str(i) for i in range(1, 21)])
        self.thread_num_input.setCurrentIndex(19)
        # Default values and placeholders
        self.ip_input.setPlaceholderText('192.168.1.0/24')
        self.port_input.setPlaceholderText('22,23,80,443,80-200')
        self.thread_num_input.setCurrentText('500')
        self.wait_time_input.setText('1')

        # Tooltips
        self.ip_input.setToolTip('IP Range e.g., 192.168.1.0/24')
        self.port_input.setToolTip('Port Range e.g., 22-100,5000')
        self.thread_num_input.setToolTip('Number of threads (1-20)')
        self.show_refused_input.setToolTip('Show refused connections')
        self.wait_time_input.setToolTip('Wait time for response (seconds)')
        self.stop_after_count_input.setToolTip('Stop scan after x open ports found')


        form_layout.addRow('IP:', self.ip_input)
        form_layout.addRow('Port:', self.port_input)
        form_layout.addRow('Thread Num:', self.thread_num_input)
        form_layout.addRow('Show Refused:', self.show_refused_input)
        form_layout.addRow('Wait Time:', self.wait_time_input)
        form_layout.addRow('Stop After:', self.stop_after_count_input)

        self.result_output = QPlainTextEdit()
        self.run_button = QPushButton('Run Scanner')
        self.run_button.clicked.connect(self.run_scanner)

        self.status_label = QLabel('Status: Ready', self)  # New status label

        layout.addLayout(form_layout)
        layout.addWidget(self.run_button)
        layout.addWidget(self.status_label)  # Add the status label to the layout
        layout.addWidget(self.result_output)

        self.setLayout(layout)
        self.output_received.connect(self.update_output)  # Connect the signal to the slot

    def show_portscanner_info(self):
        self.info_browser = QTextBrowser()
        self.info_browser.setText(self.help_from_portscan())
        self.info_browser.setMinimumWidth(500)
        self.info_browser.setMinimumHeight(400)
        self.info_browser.setOpenExternalLinks(True)
        self.info_browser.show()

    def run_scanner(self):
        # Validate the form
        if not self.ip_input.text() or not self.port_input.text() or not self.wait_time_input.text():
            error_label = QLabel('Please fill out all required fields.', self)
            error_label.setStyleSheet('color: red')
            self.layout().addWidget(error_label)
            return

        # Extract user input
        ip_str = self.ip_input.text()
        port_str = self.port_input.text()
        thread_num = int(self.thread_num_input.currentText())
        show_refused = self.show_refused_input.isChecked()
        wait_time = int(self.wait_time_input.text())
        stop_after_count_str = self.stop_after_count_input.text()
        stop_after_count = int(stop_after_count_str) if stop_after_count_str else None

        # Parse IPs and Ports using the read_ip and read_port methods
        try:
            ip_list = self.read_ip(ip_str)
            port_list = self.read_port(port_str)
        except ValueError as ve:
            error_label = QLabel(f'Invalid input: {ve}', self)
            error_label.setStyleSheet('color: red')
            self.layout().addWidget(error_label)
            return

        # Initialize and configure the PortScannerThread
        self.portScannerThread = PortScannerThread(
            ip_list, port_list, thread_num, show_refused, wait_time, stop_after_count
        )

        # Connect signals to slots
        self.portScannerThread.textReady.connect(self.update_output)
        self.portScannerThread.scanComplete.connect(self.on_scan_complete)


        # Disable the "Run Scanner" button and update the status label
        self.run_button.setEnabled(False)
        self.status_label.setText("Status: Scanning...")

        # Start the thread
        self.portScannerThread.start()
    def update_output(self, text):
        self.result_output.appendPlainText(text)

    @pyqtSlot(list)
    def on_scan_complete(self, result):
        self.result_output.appendPlainText("Scan Complete!")
        formatted_result = "\n".join([f"IP: {ip}, Port: {port}" for ip, port in result])
        self.result_output.appendPlainText(formatted_result)

        self.status_label.setText("Status: Finished")
        self.run_button.setEnabled(True)



    def on_process_finished(self):
        self.run_button.setEnabled(True)  # Re-enable run button
        self.status_label.setText('Status: Finished')  # Update status label

    def help_from_portscan(self):
        help =help_content
        return help

    from ipaddress import ip_address, ip_network

    def read_ip(self, ip_str):
        try:
            # Check if it's a single IP address
            ip = ip_address(ip_str)
            return [str(ip)]
        except ValueError:
            pass

        try:
            # Check if it's a network range in CIDR notation
            network = ip_network(ip_str)
            return [str(ip) for ip in network]
        except ValueError:
            pass

        if ip_str.startswith("[") and ip_str.endswith("]"):
            ip_str = ip_str[1:-1]
            elements = [e.strip() for e in ip_str.split(',')]
            master = []
            for each in elements:
                try:
                    sub_list = self.read_ip(each)
                    master.extend(sub_list)
                except ValueError as e:
                    print("{} is not correctly formatted".format(each))
            return master

        raise ValueError('incorrect Match')

    def read_port(self, port_str):
        ports = port_str.split(',')
        port_list = []
        for port in ports:
            if re.match('^\d+$', port):
                port_list.append(int(port))
            elif re.match('^\d+-\d+$', port):
                p_start = int(port.split('-')[0])
                p_end = int(port.split('-')[1])
                p_range = list(range(p_start, p_end + 1))
                port_list.extend(p_range)
            else:
                raise ValueError('incorrect Match')
        return port_list

if __name__ == '__main__':
    app = QApplication([])
    app.setStyle("fusion")
    gui = PortScannerGUI()
    gui.show()
    app.exec()



from PyQt6.QtCore import QObject, pyqtSignal, pyqtSlot
import serial
from serial.tools import list_ports

from uglypty.uglyplugin_serial.Library.serialshellreader import SerialReaderThread

class SerialBackend(QObject):
    send_output = pyqtSignal(str)

    def __init__(self, port='COM1', baudrate=9600, databits=8, stopbits=1, parity='N', parent=None):
        super().__init__(parent)
        self.parent = parent

        # Print available serial ports for debugging
        print("Available serial ports:")
        for port_info in list_ports.comports():
            print(f"{port_info.device}: {port_info.description}")
        self.port = port
        self.baudrate = baudrate
        self.databits = databits
        self.stopbits = stopbits
        self.parity = parity
        self.serial = None
        self.reader_thread = None

    @pyqtSlot()
    def connect(self):
        if not self.serial:
            try:
                self.serial = serial.Serial(
                    port=self.port,
                    baudrate=self.baudrate,
                    bytesize=self.databits,
                    stopbits=self.stopbits,
                    parity=self.parity,
                    timeout=1  # timeout for reading operations
                )
                print(f"Connected to... {self.port}")
            except Exception as e:
                print(e)
                return
            self.reader_thread = SerialReaderThread(self.serial)
            self.reader_thread.data_ready.connect(self.send_output)
            self.reader_thread.start()

    @pyqtSlot()
    def disconnect(self):
        if self.serial and self.serial.is_open:
            self.serial.close()
        if self.reader_thread:
            self.reader_thread.quit()
            self.reader_thread.wait()
        try:
            self.ui.close()
        except Exception as e:
            print(f"{e}")


    @pyqtSlot(str)
    def write_data(self, data):
        if self.serial and self.serial.is_open:
            try:
                self.serial.write(data.encode())
            except Exception as e:
                print(f"Error while writing to serial port: {e}")

    def __del__(self):
        self.disconnect()



from PyQt6.QtCore import pyqtSignal, pyqtSlot, QThread
from PyQt6.QtGui import QScreen, QGuiApplication
from PyQt6.QtWidgets import QApplication, QWidget, QPushButton, QLineEdit, QVBoxLayout, QLabel, QTextEdit
import sys
import subprocess

class RunThread(QThread):
    signal = pyqtSignal('PyQt_PyObject')
    stdout_signal = pyqtSignal(str)

    def __init__(self, cmd):
        QThread.__init__(self)
        self.cmd = cmd

    def run(self):
        process = subprocess.Popen(self.cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
        self.signal.emit(process)

        for line in iter(process.stdout.readline, ''):
            self.stdout_signal.emit(line)

        process.stdout.close()
        process.wait()

class crawlerGuiUI(QWidget):

    def __init__(self, vendor="cisco"):
        super(crawlerGuiUI, self).__init__()
        self.vendor = vendor
        self.initUI()


    def center(self):
        screen = QGuiApplication.primaryScreen()  # Get the primary screen
        screen_geometry = screen.availableGeometry()  # Get available geometry of the screen
        self.setGeometry(
            (screen_geometry.width() - self.width()) // 2,
            (screen_geometry.height() - self.height()) // 2,
            self.width(),
            self.height()
        )

    def initUI(self):
        try:
            vendor = self.vendor
        except:
            vendor = "cisco"
        self.setWindowTitle(f'{vendor.upper()} NDP Crawler and Mapper')
        self.setGeometry(100, 100, 400, 400)
        self.center()
        vbox = QVBoxLayout()

        self.seed_ip_edit = QLineEdit(self)
        self.seed_ip_edit.setPlaceholderText("Enter seed IP")
        vbox.addWidget(self.seed_ip_edit)

        self.device_id_edit = QLineEdit(self)
        self.device_id_edit.setPlaceholderText("Enter device ID")
        vbox.addWidget(self.device_id_edit)

        self.username_edit = QLineEdit(self)
        self.username_edit.setPlaceholderText("Enter username")
        vbox.addWidget(self.username_edit)

        self.password_edit = QLineEdit(self)
        self.password_edit.setPlaceholderText("Enter password")
        self.password_edit.setEchoMode(QLineEdit.EchoMode.Password)
        vbox.addWidget(self.password_edit)

        self.domain_name = QLineEdit(self)
        self.domain_name.setPlaceholderText("Enter domain name")
        self.domain_name.setText("company.com")
        vbox.addWidget(self.domain_name)

        self.exclude_string = QLineEdit(self)
        self.exclude_string.setPlaceholderText("comma separated string - use for network edges")
        self.exclude_string.setText("SEP,IPPhone")
        vbox.addWidget(self.exclude_string)

        self.layout_algo = QLineEdit(self)
        self.layout_algo.setPlaceholderText("rt for most networks, kk for larger networks")
        self.layout_algo.setText("rt")
        vbox.addWidget(self.layout_algo)

        self.map_name = QLineEdit(self)
        self.map_name.setPlaceholderText("name of .graphml file to save")
        self.map_name.setText("map.graphml")
        vbox.addWidget(self.map_name)

        self.collection_name = QLineEdit(self)
        self.collection_name.setPlaceholderText("collection folder to save parsed cdp/lldp info")
        self.collection_name.setText("")
        vbox.addWidget(self.collection_name)
        self.status_label = QLabel('Status: Idle', self)
        vbox.addWidget(self.status_label)

        btn1 = QPushButton('Run', self)
        btn1.clicked.connect(self.run_main)
        vbox.addWidget(btn1)

        btn2 = QPushButton('Cancel', self)
        btn2.clicked.connect(self.cancel_main)
        vbox.addWidget(btn2)

        self.output_text_edit = QTextEdit(self)
        self.output_text_edit.setReadOnly(True)
        vbox.addWidget(self.output_text_edit)

        self.setLayout(vbox)

    def run_main(self):
        seed_ip = self.seed_ip_edit.text()
        device_id = self.device_id_edit.text()
        username = self.username_edit.text()
        password = self.password_edit.text()
        domain_name = self.domain_name.text()
        exclude_string = self.exclude_string.text()
        layout_algo = self.layout_algo.text()
        map_name = self.map_name.text()
        collection_name = self.collection_name.text()
        if len(str(exclude_string).strip()) == 0:
            exclude_string = "nothing"
        cmd = [
            "cmd", "/c", sys.executable,"-u", "crawl.py",
            "--seed_ip", seed_ip,
            "--device_id", device_id,
            "--username", username,
            "--password", password,
            "--domain_name", domain_name,
            "--exclude_string", exclude_string,
            "--layout_algo", layout_algo,
            "--map_name", map_name,
            "--collection_name", collection_name,
            "--vendor", self.vendor

        ]

        self.status_label.setText("Status: Running")
        self.thread = RunThread(cmd)  # Modify the constructor of RunThread to accept cmd directly
        self.thread.signal.connect(self.set_process)
        self.thread.stdout_signal.connect(self.update_text_edit)
        self.thread.start()

    def set_process(self, process):
        self.process = process

    def update_text_edit(self, text):
        self.output_text_edit.append(text.replace("\n", ""))

    def cancel_main(self):
        self.process.terminate()
        self.status_label.setText("Status: Terminated")

if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setStyle("fusion")
    ex = crawlerGuiUI()
    ex.resize(600, 500)
    ex.show()
    sys.exit(app.exec())



p_examples = {
    "TTP": {
        'teSource': """interface Loopback0
 description Router-id-loopback
 ip address 192.168.0.113/24
!
interface Vlan778
 description CPE_Acces_Vlan
 ip address 10.28.0.200/24
 ip vrf DATA
!""",
        'teTemplate': """interface {{ interface }}
 ip address {{ ip }}/{{ mask }}
 description {{ description }}
 ip vrf {{ vrf }}"""
    },
    "Jinja2": {
        'teSource': """---
isis:
  instancetag: qa-test
  net: 49.0001.1720.1600.1001.00
  level: level-1
  authtype: md5
  key: cisco
  interfaces:
  - name: port-channel313
""",
        'teTemplate': """feature isis
!
router isis {{isis.instancetag}}
  net {{isis.net}}
  is-type {{isis.level}}
  log-adjacency-changes
  authentication-type {{isis.authtype}} {{isis.level}}
  authentication key-chain {{isis.key}} {{isis.level}}
  address-family ipv4 unicast
    bfd
  address-family ipv6 unicast
    bfd
!
{% for interface in isis.interfaces -%}
interface {{interface.name}}
  ip router isis {{isis.instancetag}}
  ipv6 router isis {{isis.instancetag}}
{% endfor -%}
!
"""
    },
    "JMesPath": {
        'teSource': """[
    {
        "ntp_associations_detail": [
            {
                "peer": "10.240.72.125",
                "reach": "377",
                "root_delay": "1541.1377",
                "root_disp": "34088.13",
                "stratum": "2",
                "sync_dist": "55841.0566",
                "type": "candidate"
            },
            {
                "peer": "2605:1c00:50f3:70:10:240:72:125",
                "reach": "377",
                "root_delay": "1541.1377",
                "root_disp": "22109.99",
                "stratum": "2",
                "sync_dist": "55537.1848",
                "type": "candidate"
            },
            {
                "peer": "172.30.105.134",
                "reach": "377",
                "root_delay": "0.0000",
                "root_disp": "991.82",
                "stratum": "1",
                "sync_dist": "31185.3384",
                "type": "our_master"
            }
        ]
    }
]
""",
        'teTemplate': """[0].ntp_associations_detail[?peer == '172.30.105.134']"""
    }

}


from PyQt6.QtCore import QThread, pyqtSignal
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QTextEdit, QPushButton, QLineEdit, QLabel, QFormLayout, \
    QComboBox
import time
import socket
import paramiko
import threading
import logging

from uglypty.uglyplugin_sftp.sftp_lib import StubServer, StubSFTPServer

import secrets
import string

def generate_random_password(length=10):
    alphabet = string.ascii_letters + string.digits
    return ''.join(secrets.choice(alphabet) for i in range(length))
class SFTPServerThread(QThread):
    logSignal = pyqtSignal(str)

    def __init__(self, host, port, keyfile, level):
        super(SFTPServerThread, self).__init__()
        self.host = host
        self.port = port
        self.keyfile = keyfile
        self.level = level
        self.stop_flag = threading.Event()

        paramiko_logger = logging.getLogger("paramiko")
        paramiko_logger.setLevel(getattr(logging, self.level))
        handler = PyQtLogHandler(self.logSignal)
        paramiko_logger.addHandler(handler)

    def run(self):
        # Generate random password
        alphabet = string.ascii_letters + string.digits
        self.random_password = ''.join(secrets.choice(alphabet) for i in range(10))

        # You could add code here to set the password to the Paramiko authentication
        # For example, self.set_password(random_password)

        # Notify through logs
        logging.info(f"Creds: admin/{self.random_password}")
        self.logSignal.emit(f"Creds: admin/{self.random_password}")

        self.start_server()

    def handle_connection(self, conn, addr):
        logging.info(f"New connection from {addr}")
        self.logSignal.emit(f"New connection from {addr}")

        host_key = paramiko.RSAKey.from_private_key_file(self.keyfile)
        transport = paramiko.Transport(conn)
        transport.add_server_key(host_key)
        transport.set_subsystem_handler('sftp', paramiko.SFTPServer, StubSFTPServer)
        # random password set in run() for display
        server = StubServer("admin", self.random_password)
        transport.start_server(server=server)

        channel = transport.accept()
        while transport.is_active():
            time.sleep(1)

        logging.info(f"Connection from {addr} closed")
        self.logSignal.emit(f"Connection from {addr} closed")

    def start_server(self):
        paramiko_level = getattr(paramiko.common, self.level)
        paramiko.common.logging.basicConfig(level=paramiko_level)

        logging.info(f"Starting SFTP server on {self.host}:{self.port} with keyfile {self.keyfile}")
        self.logSignal.emit(f"Starting SFTP server on {self.host}:{self.port} with keyfile {self.keyfile}")

        server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, True)
        server_socket.bind((self.host, self.port))
        server_socket.listen(10)

        while not self.stop_flag.is_set():
            conn, addr = server_socket.accept()
            threading.Thread(target=self.handle_connection, args=(conn, addr)).start()

        # Close the socket after loop termination (initiated by stop request)
        server_socket.close()


class PyQtLogHandler(logging.Handler):
    def __init__(self, signal):
        super().__init__()
        self.signal = signal

    def emit(self, record):
        log_entry = self.format(record)
        self.signal.emit(log_entry)


class SFTPWidget(QWidget):
    def __init__(self, parent=None):
        super(SFTPWidget, self).__init__()
        self.parent = parent
        self.setWindowTitle("SFTP Server")
        self.logArea = QTextEdit()
        self.logArea.setMinimumWidth(500)
        self.logArea.setReadOnly(True)
        self.startButton = QPushButton('Start Server')
        self.startButton.setStyleSheet("background-color: #006400; color: white;")
        self.stopButton = QPushButton('Stop Server')
        self.stopButton.setStyleSheet("background-color: #8B0000; color: white;")
        self.stopButton.setEnabled(False)
        self.stop_flag = threading.Event()

        self.hostInput = QLineEdit('0.0.0.0')
        self.portInput = QLineEdit('2022')
        self.keyfileInput = QLineEdit('./test_rsa.key')
        # self.levelInput = QLineEdit('DEBUG')
        self.levelInput = QComboBox()
        self.levelInput.addItems(["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"])
        self.levelInput.setStyleSheet("background-color: #8B6508; color: white;")
        self.levelInput.setCurrentIndex(3)
        layout = QVBoxLayout()
        formLayout = QFormLayout()
        formLayout.addRow(QLabel('Listening IP:'), self.hostInput)
        formLayout.addRow(QLabel('Port:'), self.portInput)
        formLayout.addRow(QLabel('Keyfile:'), self.keyfileInput)
        formLayout.addRow(QLabel('Log Level:'), self.levelInput)

        layout.addLayout(formLayout)
        layout.addWidget(self.startButton)
        layout.addWidget(self.stopButton)
        layout.addWidget(self.logArea)

        self.setLayout(layout)

        self.startButton.clicked.connect(self.start_server)
        self.stopButton.clicked.connect(self.stop_server)

    def start_server(self):

        self.sftp_thread = SFTPServerThread(self.hostInput.text(), int(self.portInput.text()), self.keyfileInput.text(),
                                            self.levelInput.currentText())
        self.sftp_thread.logSignal.connect(self.update_log)
        self.sftp_thread.start()
        self.startButton.setEnabled(False)
        self.stopButton.setEnabled(True)
        self.stop_flag.clear()

    def stop_server(self):
        # Implement stop logic
        self.startButton.setEnabled(True)
        self.stopButton.setEnabled(False)
        self.stop_flag.set()
        self.logArea.append("\nServer stopped")

    def update_log(self, log_message):
        self.logArea.append(log_message)

if __name__ == "__main__":
    app = QApplication([])
    window = SFTPWidget()
    window.show()
    app.exec()



import json
from N2G import yed_diagram

class NetworkDiagramGenerator:

    def __init__(self, json_file_path, domain_name):
        self.json_file_path = json_file_path
        self.domain_name = domain_name
        self.diagram = yed_diagram()
        self.network_map = self._load_network_map()

    def _load_network_map(self):
        with open(self.json_file_path, 'r') as f:
            return json.load(f)

    @staticmethod
    def _clean_device_id(device_id, domain):
        return device_id.replace(domain, "")



    def generate(self):
        for device, connections in self.network_map.items():
            cleaned_device = self._clean_device_id(device, self.domain_name)
            platform = connections[0].get('platform', '')
            ip = connections[0].get('ip', 'Unknown')
            self.diagram.add_node(cleaned_device, top_label=ip, bottom_label=platform)

            for conn in connections:
                try:
                    neighbor = conn['device_id']
                    cleaned_neighbor = self._clean_device_id(neighbor, self.domain_name)
                    neighbor_platform = conn.get('platform', '')
                    neighbor_ip = conn.get('ip', '')

                    if cleaned_neighbor not in [self._clean_device_id(dev, self.domain_name) for dev in
                                                self.network_map.keys()]:
                        self.diagram.add_node(cleaned_neighbor, top_label=neighbor_ip, bottom_label=neighbor_platform)

                    local_port = conn.get('local_port', 'Unknown')
                    remote_port = conn.get('remote_port', 'Unknown')
                    self.diagram.add_link(cleaned_device, cleaned_neighbor, label='', src_label=local_port,
                                          trgt_label=remote_port)
                except Exception as e:
                    print(f"Error processing connection: {e}")
    def layout_and_save(self, layout_algo="kk", output_folder="./Output/", filename="Network_graph.graphml"):
        self.diagram.layout(algo=layout_algo)
        self.diagram.dump_file(filename=filename, folder=output_folder)


if __name__ == "__main__":
    generator = NetworkDiagramGenerator('./Output/network_map.json', ".company.com")
    generator.generate()
    generator.layout_and_save(layout_algo="rt") # kk, rt



import json

from PyQt6.QtCore import Qt, pyqtSignal, QThread
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QLineEdit, QPushButton, QProgressBar, QLabel, \
    QFileDialog
import pynetbox
import yaml
import urllib3

# Suppress only the single InsecureRequestWarning from urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class DownloadThread(QThread):
    progress_signal = pyqtSignal(int)
    status_signal = pyqtSignal(str)
    save_file_signal = pyqtSignal(object)

    def __init__(self, token, url):
        QThread.__init__(self)
        self.token = token
        self.url = url

    def run(self):
        try:
            self.status_signal.emit("Status: Connecting to Netbox")
            netbox = pynetbox.api(self.url, token=self.token)
            netbox.http_session.verify = False  # This is insecure; use with caution

            self.status_signal.emit("Status: Downloading Data")
            uglypty_list = []

            # First loop to count total sites
            site_list = []
            sites = netbox.dcim.sites.all()
            for temp_site in sites:
                print(f"Site Preloading: {temp_site.name}")
                site_list.append(temp_site)
            total_sites = len(site_list)

            # Reset progress bar
            counter = 0

            # Second loop to actually get the data
            for site in site_list:
                folder_dict = {}
                folder_dict['folder_name'] = site.slug
                folder_dict['sessions'] = []
                devices = netbox.dcim.devices.filter(site_id=site.id)

                for device in devices:
                    session = {}
                    try:
                        session['DeviceType'] = device.device_role.name if device.device_role else 'Unknown'
                        session['Model'] = device.device_type.model
                        session['SerialNumber'] = device.serial
                        session['SoftwareVersion'] = 'Unknown'  # Replace with actual data if available
                        session[
                            'Vendor'] = device.device_type.manufacturer.name if device.device_type.manufacturer else 'Unknown'
                        session['credsid'] = '1'
                        session['display_name'] = device.name
                        session['host'] = str(device.primary_ip4.address).split("/")[0] if device.primary_ip4 else 'Unknown'
                        session['port'] = '22'

                        folder_dict['sessions'].append(session)
                        print(f"Processed: {device.name}")
                    except Exception as e:
                        print(f"Error processing device: {device.name}")
                        print(e)

                uglypty_list.append(folder_dict)
                counter += 1
                print(f"Site Number: {counter} of {total_sites}")

                # Update the progress bar
                self.progress_signal.emit((counter * 100) // total_sites)
            # save successfull settings
            settings = {
                "token": self.token,
                "url": self.url
            }
            try:
                with open("netbox_settings.json", "w") as f:
                    json.dump(settings, f)
            except Exception as e:
                print(f"failed to save netbox settings! {e}")
            # Save the YAML file
            # self.status_signal.emit("Status: Saving YAML File")
            # with open("netbox_sessions.yaml", "w") as f:
            #     yaml.dump(uglypty_list, f, default_flow_style=False)
            self.save_file_signal.emit(uglypty_list)


            self.status_signal.emit("Download Complete. Saved to 'netbox_sessions.yaml'")
            self.progress_signal.emit(100)

        except Exception as e:
            print("An error occurred:", e)
            self.status_signal.emit("Status: An error occurred")

def create_default_settings():
    default_settings = {
        "token": "",
        "url": "http://netbox.yourcompany.com"
    }
    with open("netbox_settings.json", "w") as f:
        json.dump(default_settings, f)
class App(QWidget):
    def __init__(self, parrent=None):
        super().__init__()
        self.parent = parrent
        self.title = 'Netbox to UglyPTY YAML Exporter'
        # Load Netbox settings from JSON if exists, else create it.
        try:
            with open("netbox_settings.json", "r") as f:
                settings = json.load(f)
            self.default_token = settings.get("token", "")
            self.default_url = settings.get("url", "http://netbox.yourcompany.com")
        except FileNotFoundError:
            create_default_settings()
            self.default_token = ""
            self.default_url = "http://netbox.yourcompany.com"
        self.initUI()

    def initUI(self):
        layout = QVBoxLayout()
        self.setWindowTitle("Netbox to Session Export")
        self.setMinimumWidth(300)
        self.tokenField = QLineEdit(self)
        self.tokenField.setText("")
        self.tokenField.setPlaceholderText('Enter your Netbox Token')
        self.tokenField.setText(self.default_token)
        layout.addWidget(self.tokenField)

        self.urlField = QLineEdit(self)
        self.urlField.setText("http://netbox.yourcompany.com")
        self.urlField.setPlaceholderText('Enter your Netbox URL')
        self.urlField.setText(self.default_url)
        layout.addWidget(self.urlField)

        self.downloadButton = QPushButton('Download', self)
        self.downloadButton.clicked.connect(self.startDownloadThread)
        self.downloadButton.setStyleSheet("background-color: #006400; color: white;")
        layout.addWidget(self.downloadButton)

        self.progress = QProgressBar(self)
        layout.addWidget(self.progress)

        self.statusLabel = QLabel('Status: Waiting', self)
        layout.addWidget(self.statusLabel)

        self.setLayout(layout)
        self.show()

    def startDownloadThread(self):
        self.downloadThread = DownloadThread(self.tokenField.text(), self.urlField.text())
        self.downloadThread.progress_signal.connect(self.updateProgressBar)
        self.downloadThread.status_signal.connect(self.updateStatusLabel)
        self.downloadThread.save_file_signal.connect(self.showSaveFileDialog)
        self.downloadThread.start()

    def showSaveFileDialog(self, uglypty_list):

        filePath, _ = QFileDialog.getSaveFileName(self, "Save File", "", "YAML Files (*.yaml);;All Files (*)")
        if filePath:
            if not filePath.endswith('.yaml'):
                filePath += '.yaml'
            with open(filePath, "w") as f:
                yaml.dump(uglypty_list, f, default_flow_style=False)
            self.statusLabel.setText(f"Download Complete. Saved to {filePath}")
    def updateProgressBar(self, value):
        self.progress.setValue(value)

    def updateStatusLabel(self, text):
        self.statusLabel.setText(f"Status: {text}")



if __name__ == '__main__':
    app = QApplication([])
    ex = App()
    app.exec()



from PyQt6.QtCore import Qt
from PyQt6.QtWidgets import QApplication, QWidget, QVBoxLayout, QTabWidget, QLineEdit, QLabel, QPushButton, \
    QTableWidget, QTableWidgetItem, QFormLayout, QFileDialog
from ipaddress import ip_network, ip_interface
import csv
import sys


class MyWindow(QWidget):
    def __init__(self):
        super(MyWindow, self).__init__()

        self.setWindowTitle("IP Subnet Calculator")
        self.setGeometry(200, 200, 600, 400)

        layout = QVBoxLayout()
        tab_widget = QTabWidget()

        # IPv4 Tab
        tab1 = QWidget()
        tab1_layout = QVBoxLayout()
        self.setup_ipv4_tab(tab1, tab1_layout)
        tab_widget.addTab(tab1, "IPv4")

        # IPv6 Tab
        tab2 = QWidget()
        tab2_layout = QVBoxLayout()
        self.setup_ipv6_tab(tab2, tab2_layout)
        tab_widget.addTab(tab2, "IPv6")

        layout.addWidget(tab_widget)
        self.setLayout(layout)

    def setup_ipv4_tab(self, tab, layout):
        form_layout = self.create_form_layout('ipv4')
        calc_button = QPushButton('Calculate')
        calc_button.clicked.connect(lambda: self.calculate_subnets('ipv4'))
        form_layout.addRow('', calc_button)

        self.summary_text_ipv4 = QLabel('')
        form_layout.addRow('Summary:', self.summary_text_ipv4)

        self.table_ipv4 = self.create_table()
        layout.addLayout(form_layout)
        layout.addWidget(self.table_ipv4)

        export_button = QPushButton('Export to CSV')
        export_button.clicked.connect(lambda: self.export_table_to_csv('ipv4'))
        layout.addWidget(export_button)

        tab.setLayout(layout)

    def setup_ipv6_tab(self, tab, layout):
        form_layout = self.create_form_layout('ipv6')
        calc_button_ipv6 = QPushButton('Calculate IPv6')
        calc_button_ipv6.clicked.connect(lambda: self.calculate_subnets('ipv6'))
        form_layout.addRow('', calc_button_ipv6)

        self.summary_text_ipv6 = QLabel('')
        form_layout.addRow('Summary:', self.summary_text_ipv6)

        self.table_ipv6 = self.create_table()
        layout.addLayout(form_layout)
        layout.addWidget(self.table_ipv6)

        export_button_ipv6 = QPushButton('Export to CSV (IPv6)')
        export_button_ipv6.clicked.connect(lambda: self.export_table_to_csv('ipv6'))
        layout.addWidget(export_button_ipv6)

        tab.setLayout(layout)

    def create_form_layout(self, ip_version):
        form_layout = QFormLayout()
        setattr(self, f'ip_input_{ip_version}', QLineEdit())
        setattr(self, f'prefix_input_{ip_version}', QLineEdit())
        setattr(self, f'subnet_prefix_input_{ip_version}', QLineEdit())
        form_layout.addRow('IP Prefix:', getattr(self, f'ip_input_{ip_version}'))
        form_layout.addRow('Prefix Length:', getattr(self, f'prefix_input_{ip_version}'))
        form_layout.addRow('Subnet Prefix Length:', getattr(self, f'subnet_prefix_input_{ip_version}'))
        return form_layout

    def create_table(self):
        table = QTableWidget()
        table.setColumnCount(5)
        table.setHorizontalHeaderLabels(
            ['Subnet', 'Prefix Length', 'Starting Host IP', 'Ending Host IP', 'Broadcast Address'])
        return table

    def calculate_subnets(self, ip_version):
        # ip_prefix = f'ip_input_{ip_version}'
        ip_prefix = getattr(self, f'ip_input_{ip_version}').text()
        try:
            prefix_length = int(getattr(self, f'prefix_input_{ip_version}').text())
            subnet_prefix_length = int(getattr(self, f'subnet_prefix_input_{ip_version}').text())
            network = ip_network(f"{ip_prefix}/{prefix_length}", strict=False)
        except ValueError as e:
            if ip_version == "ipv4":
                self.summary_text_ipv4.setText(f"Calculation Error: {str(e)}")
            if ip_version == "ipv6":
                self.summary_text_ipv6.setText(f"Calculation Error: {str(e)}")
            return


        subnets = list(network.subnets(new_prefix=subnet_prefix_length))

        summary_text = f"Number of Subnets: {len(subnets)}, Hosts per Subnet: {len(list(subnets[0].hosts()))}"
        summary_label = getattr(self, f'summary_text_{ip_version}')
        summary_label.setText(summary_text)

        table = getattr(self, f'table_{ip_version}')
        table.setRowCount(len(subnets))

        for idx, subnet in enumerate(subnets):
            hosts = list(subnet.hosts())
            table.setItem(idx, 0, QTableWidgetItem(str(subnet.network_address)))
            table.setItem(idx, 1, QTableWidgetItem(str(subnet.prefixlen)))
            table.setItem(idx, 2, QTableWidgetItem(str(hosts[0]) if hosts else 'N/A'))
            table.setItem(idx, 3, QTableWidgetItem(str(hosts[-1]) if hosts else 'N/A'))
            table.setItem(idx, 4, QTableWidgetItem(str(subnet.broadcast_address)))

    def export_table_to_csv(self, ip_version):
        table = getattr(self, f'table_{ip_version}')
        default_filename = f"{ip_version}_subnets.csv"

        filename, _ = QFileDialog.getSaveFileName(self, "Save File", default_filename,
                                                  "CSV Files (*.csv);;All Files (*)")

        if not filename:  # Cancelled or closed dialog
            return

        # Add a .csv extension if not present
        if not filename.endswith('.csv'):
            filename += '.csv'

        try:
            with open(filename, 'w', newline='') as csvfile:
                writer = csv.writer(csvfile)

                # Write the header
                headers = [table.horizontalHeaderItem(i).text() for i in range(table.columnCount())]
                writer.writerow(headers)

                # Write the table data
                for row in range(table.rowCount()):
                    row_data = []
                    for column in range(table.columnCount()):
                        item = table.item(row, column)
                        if item is not None:
                            row_data.append(item.text())
                        else:
                            row_data.append('')
                    writer.writerow(row_data)

        except Exception as e:
            summary_label = getattr(self, f'summary_text_{ip_version}')
            summary_label.setText(f"Save Error: {str(e)}")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    app.setStyle("fusion")
    window = MyWindow()
    window.show()
    sys.exit(app.exec())



import re
from PyQt6.QtGui import QColor, QTextCharFormat, QFont, QSyntaxHighlighter
from PyQt6.QtCore import Qt

class SyntaxHighlighter(QSyntaxHighlighter):

    def __init__(self, document):
        QSyntaxHighlighter.__init__(self, document)

        # Keywords
        self.keywords = []
        self.syntax_type = None  # Added syntax type attribute

    def load_keywords_from_file(self, file_path):
        with open(file_path, 'r') as file:
            self.keywords = [line.strip() for line in file]

    def set_syntax_type(self, syntax_type):  # Added setter for syntax type
        self.syntax_type = syntax_type

    def highlightBlock(self, text):
        if self.syntax_type == "keyword":
            self.highlight_keywords(text)
        elif self.syntax_type == "yaml":
            self.highlight_yaml(text)
        elif self.syntax_type == "json":
            self.highlight_json(text)
        elif self.syntax_type == "jinja":
            self.highlight_jinja(text)

    def highlight_keywords(self, text):
        format = QTextCharFormat()
        format.setFontWeight(QFont.Weight.Bold)
        format.setForeground(Qt.GlobalColor.darkMagenta)

        for keyword in self.keywords:
            pattern = f"\\b{keyword}\\b"
            self.highlight_pattern(text, pattern, format)

    def highlight_yaml(self, text):
        format = QTextCharFormat()
        format.setFontWeight(QFont.Weight.Bold)
        format.setForeground(Qt.GlobalColor.green)
        pattern = r"\b[a-zA-Z_][a-zA-Z0-9_]*\b(?=\s*:)"  # Match keys before colon
        self.highlight_pattern(text, pattern, format)

    def highlight_json(self, text):
        format = QTextCharFormat()
        format.setFontWeight(QFont.Weight.Bold)
        format.setForeground(Qt.GlobalColor.yellow)
        pattern = r"\".*?\": "  # Match key-value pair structure of JSON
        self.highlight_pattern(text, pattern, format)

    def highlight_jinja(self, text):
        format1 = QTextCharFormat()
        format1.setFontWeight(QFont.Weight.Bold)
        format1.setForeground(Qt.GlobalColor.darkCyan)
        format1.setUnderlineStyle(QTextCharFormat.UnderlineStyle.SingleUnderline)
        pattern1 = r"\{\{.*?\}\}"  # Match Jinja2-style variable syntax
        self.highlight_pattern(text, pattern1, format1)

        format2 = QTextCharFormat()
        format2.setFontWeight(QFont.Weight.Bold)
        format2.setForeground(Qt.GlobalColor.green)
        pattern2 = r"\{%\s*for.*?%\}"  # Match Jinja2-style for loop syntax
        self.highlight_pattern(text, pattern2, format2)

        format3 = QTextCharFormat()
        format3.setFontWeight(QFont.Weight.Bold)
        format3.setForeground(Qt.GlobalColor.blue)
        pattern3 = r"\{%\s*(if|elif|else).*?%\}"  # Match Jinja2-style if/elif/else syntax
        self.highlight_pattern(text, pattern3, format3)

        format4 = QTextCharFormat()
        format4.setFontWeight(QFont.Weight.Bold)
        format4.setForeground(Qt.GlobalColor.green)
        pattern4 = r"\{%\s*end(for|if).*?%\}"  # Match Jinja2-style endfor/endif syntax
        self.highlight_pattern(text, pattern4, format4)

    def highlight_pattern(self, text, pattern, format):
        expression = re.compile(pattern)
        index = expression.search(text)

        while index:
            start, end = index.span()
            length = end - start
            self.setFormat(start, length, format)
            index = expression.search(text, start + length)

if __name__ == '__main__':
    import sys
    from PyQt6.QtWidgets import QApplication, QTextEdit

    app = QApplication(sys.argv)

    text_edit = QTextEdit()
    highlighter = SyntaxHighlighter(text_edit.document())
    highlighter.set_syntax_type("jinja")
    text_edit.show()

    sys.exit(app.exec())



from setuptools import setup, find_packages

setup(
    name="uglypty",
    version="0.8.12",
    description="UglyPTY - A PyQt6-based SSH, Mapping and Automation Tooling.",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    author="Scott Peterman",
    author_email="scottpeterman@gmail.com",
    url="https://github.com/scottpeterman/UglyPTY", 
    license="GPLv3",
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=['asttokens==2.2.1',
                     'backcall==0.2.0',
                     'bcrypt==4.0.1',
                     'cached-property==1.5.2',
                     'certifi==2023.7.22',
                     'cffi==1.15.1',
                     'charset-normalizer==3.2.0',
                     'click==8.1.3',
                     'colorama==0.4.6',
                     'crayons==0.4.0',
                     'cryptography==41.0.1',
                     'darkdetect==0.7.1',
                     'decorator==5.1.1',
                    'diff-match-patch==20230430',
                     'executing==1.2.0',
                     'future==0.18.3',
                     'greenlet==2.0.2',
                     'idle==1.0.4',
                     'idna==3.4',
                     'igraph==0.10.6',
                     'inflection==0.5.1',
                     'jedi==0.19.0',
                     'Jinja2==3.1.2',
                     'jmespath==1.0.1',
                      'ldap3==2.9.1',
                     'MarkupSafe==2.1.3',
                     'matplotlib-inline==0.1.6',
                     'mypy-extensions==1.0.0',
                      "netmiko==3.3.3",
                     'numpy==1.25.2',
                     'n2g==0.3.3',
                     'pandas==2.0.3',
                     'pandasql==0.7.3',
                     'paramiko==3.2.0',
                     'parso==0.8.3',
                     'pickleshare==0.7.5',
                     'prompt-toolkit==3.0.39',
                     'pure-eval==0.2.2',
                     'pycparser==2.21',
                     'pycryptodome==3.18.0',
                     'Pygments==2.16.1',
                     'PyNaCl==1.5.0',
                     'PyQt6==6.5.1',
                     'PyQt6-Qt6==6.5.1',
                     'PyQt6-sip==13.5.1',
                     'PyQt6-WebEngine==6.5.0',
                     'PyQt6-WebEngine-Qt6==6.5.1',
                     'pyqtdarktheme==2.1.0',
                     'pyserial==3.5',
                     'python-dateutil==2.8.2',
                     'python-igraph==0.10.6',
                     'pytz==2023.3',
                      'pynetbox==7.2.0',
                     'PyYAML==6.0.1',
                     'qt-material==2.14',
                     'requests==2.31.0',
                     'scp==0.14.5',
                     'six==1.16.0',
                     'SQLAlchemy==2.0.20',
                     'sqlalchemy-orm==1.2.10',
                     'SQLAlchemy-serializer==1.4.1',
                     'stack-data==0.6.2',
                     'svgwrite==1.4.3',
                     'textfsm==1.1.3',
                     'texttable==1.6.7',
                     'tomli==2.0.1',
                     'tqdm==4.66.1',
                    'tftpy==0.8.2',
                      'ttp==0.9.5',
                     'traitlets==5.9.0',
                     'typing-inspect==0.9.0',
                     'typing_extensions==4.7.1',
                     'tzdata==2023.3',
                     'urllib3==2.0.4',
                     'wcwidth==0.2.6'],
    #
    #
# scripts=['uglypty/uglypty.py'],
    package_data={
        'uglypty': [
             'icons/*',
        'Library/*',
        'logs/*',
        'static/*',
        'static/images/*',
        'sessions/*',
        'uglyplugin_serial/*',
        '*.html'

        ],
    },
    python_requires='>=3.9',
)



#!/usr/local/opt/python/bin/python3.7
import socket
import threading
import argparse
import re
import os
import time

from queue import Queue
from queue import Empty
try:
    import resource
    # Expand thread number possible with extended FILE count.
    # This remain as low as 2048 due to macOS secret open limit, unfortunately
    resource.setrlimit(resource.RLIMIT_NOFILE, (2048, 2048))
except ModuleNotFoundError:
    pass  # for windows support, skip this limitation


# A multithreading portscan module
class PortScan:

    # Regex Strings for parsing
    SINGLE_IP = r'^(?:\d{1,3}\.){3}\d{1,3}$'
    BLOCK_24 = r'^(?:\d{1,3}\.){3}0\/24$'
    GROUPED_IP = r'^\[.*\]$'

    def __init__(self, ip_str, port_str=None, thread_num=500, show_refused=False, wait_time=3, stop_after_count=None):
        self.ip_range = self.read_ip(ip_str)
        if port_str is None:
            self.ports = [22, 23, 80, 443]
        else:
            self.ports = self.read_port(port_str)
        self.lock = threading.RLock()
        self.thread_num = thread_num
        if self.thread_num > 2047:
            self.thread_num = 2047
        self.q = Queue(maxsize=self.thread_num*3)
        self.gen = None # Generator instance to be instantiated later
        self.show_refused = show_refused
        if wait_time <= 0:
            raise "Cannot have negative or zero wait time"
        self.wait_time = wait_time
        self.stop_after_count = stop_after_count
        self.ping_counter = 0

    # Read in IP Address from string.
    def read_ip(self, ip_str):
        # Single IP address
        if re.match(PortScan.SINGLE_IP, ip_str):
            if all([x<256 for x in map(int, ip_str.split('.'))]):
                return [ip_str]
            raise ValueError('incorrect IP Address')
        # Block 24 IP address.
        if re.match(PortScan.BLOCK_24, ip_str):
            block_3 = list(map(int, ip_str.split('.')[:3]))
            if all([x<256 for x in block_3]):
                block_3s = '.'.join(map(str, block_3))
                return [block_3s+'.'+str(i) for i in range(256)]
            raise ValueError('incorrect IP Address')
        # List of IP Address
        if re.match(PortScan.GROUPED_IP, ip_str):
            ip_str = ip_str[1:-1]
            elements = [e.strip() for e in ip_str.split(',')]
            master = []
            for each in elements:
                try:
                    sub_list = self.read_ip(each)
                    master.extend(sub_list)
                except ValueError as e:
                    print("{} is not correctly formatted".format(each))
            return master
        raise ValueError('incorrect Match')

    # Read in port range from string delimited by ','
    def read_port(self, port_str):
        ports = port_str.split(',')
        port_list = []
        for port in ports:
            if re.match('^\d+$', port):
                port_list.append(int(port))
            elif re.match('^\d+-\d+$', port):
                p_start = int(port.split('-')[0])
                p_end = int(port.split('-')[1])
                p_range = list(range(p_start, p_end+1))
                port_list.extend(p_range)
            else:
                raise ValueError('incorrect Match')
        return port_list

    # Standalone thread for queue
    def fill_queue(self):
        while True:
            if self.stop_after_count is not None and self.stop_after_count <= 0:
                return # Found satisfying number of open ports, stop populating queue
            if not self.q.full():
                try:
                    self.q.put(next(self.gen))
                    self.ping_counter += 1
                except StopIteration:
                    # Break condition
                    break
            else:
                time.sleep(0.01)

    def worker(self):
        # Worker threads that take ports from queue and consume it
        while True: # never stop working!
            try:
                work = self.q.get()
                self.ping_port(*work)
            except Empty:
                return
            finally:
                self.q.task_done()

    def ping_port(self, ip, port):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(self.wait_time)
        status = sock.connect_ex((ip, port))
        if status == 0:
            with self.lock:
                print('{}:{} OPEN'.format(ip, port))
                self.open_results.append((ip,port))
                if self.stop_after_count is not None:
                    self.stop_after_count -= 1
        elif status not in [35, 64, 65] and self.show_refused:
            with self.lock:
                print('{}:{} ERRNO {}, {}'.format(ip, port, status, os.strerror(status)))
        return

    def run(self):
        # Generator that contains all ip:port pairs.
        self.gen = ((ip, port) for ip in self.ip_range for port in self.ports)
        queue_thread = threading.Thread(target=self.fill_queue)
        queue_thread.daemon = True
        queue_thread.start()
        self.open_results = []
        for i in range(self.thread_num):
            t = threading.Thread(target=self.worker)
            t.daemon = True
            t.start()
        self.q.join()
        print("Found {} ports".format(self.ping_counter))
        return self.open_results


def get_local_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP


IP_HELP_STR = """
[string] Positional Argument, if not provided, will default to your local IP address.
Accepts Single IP (e.g 172.28.31.227)
Multiple IP in a list delineated by "," and enclosed in [] (e.g. [172.28.31.227,172.28.31.228])
24 IP BLOCK (e.g. 172.28.31.0/24)
"""

PORT_HELP_STR = """
[string] range of ports, default to 22,23,80,443
accept individual ports and ranges, delineated by "," e.g: 22,80,8000-8010
"""

THREAD_HELP_STR = """
[int] maximum number of threads, default is 500, maxed at 2047 for safety.
"""

SHOW_REFUSED_HELP_STR = """
[boolean flag] Show connection that was responded to but returned a refusal.
"""

WAIT_HELP_STR = """
[float] Wait time for response, for local, this can be as low as 0.1, unit in second
"""

STOP_AFTER_HELP_STR = """
[int] Stopping after x many open port has been found, not on by default. Note that the numbers are not exact as threads will continue to finish the existing queue before exiting.
"""


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('ip', nargs='?', default=None, help=IP_HELP_STR)
    parser.add_argument('-p', '--port', action='store', dest='port', help=PORT_HELP_STR)
    parser.add_argument('-t', '--threadnum', action='store', dest='threadnum', default=500, type=int, help=THREAD_HELP_STR)
    parser.add_argument('-e', '--show_refused', action='store_true', dest='show_refused', default=False, help=SHOW_REFUSED_HELP_STR)
    parser.add_argument('-w', '--wait', action='store', dest='wait_time', default=3, type=float, help=WAIT_HELP_STR)
    parser.add_argument('-s', '--stop_after', action='store', dest='stop_after_count', default=None, type=int, help=STOP_AFTER_HELP_STR)
    args = parser.parse_args()
    if args.ip is None:
        print("No IP string found, using local address")
        ip = get_local_ip()
        print("Local IP found to be {}, scanning entire block".format(ip))
        ipblocks = ip.split('.')
        ipblocks[-1] = '0/24'
        ipfinal = '.'.join(ipblocks)
        args.ip = ipfinal
    scanner = PortScan(ip_str=args.ip, port_str=args.port,
                       thread_num=args.threadnum, show_refused=args.show_refused,
                       wait_time=args.wait_time, stop_after_count=args.stop_after_count)
    print("Threads will wait for ping response for {} seconds".format(args.wait_time))
    scanner.run()


if __name__ == '__main__':
    main()